# MakeSurfTools.tcl: a set of procedures for surface analysis
#
# Written by:
# Timothy M. Ellmore, LBC/NIMH/NIH, October 1998 to April 1999 
#
# With lots of help, suggestions and encouragement from:
# Michael S. Beauchamp, Ph.D, LBC/NIMH/NIH
# John D. Van Horn, Ph.D, LBC/NIMH/NIH
#
#
# MakeSurfTools.tcl is a library of procedures used in 
# the automation of the Dale, Fischl and Sereno surface 
# analysis scheme. 
#
# MakeSurfTools.tcl was last modified on: April 17, 1999


# MakeSurfTools.tcl needs access to the following languages/interpreters
#
# - tcl/tk (version 7.4 or higher): specifically the tclsh and wish 
#   should be in /usr/local/bin
#
# - perl (version 5.004 or higher): this is needed by the MNI script
#   "mritotal" which is called to perform an automated spatial 
#    normalization of the anatomy to the 305 MNI average brain
#


# MakeSurfTools.tcl calls the following binaries and scripts
#

# - bshortconv (C program)
# - mri_convert (C program)
# - mritotal (Perl script, calls MNI code)
# - mri_add_xform_to_header (C program)
# - mri_normalize (C program)
# - cpinfo (C program)
# - mri_strip_skull  (C program)
# - wmfilter (C program)
# - mri_fill (C program)
# - tksurfer (tk application)
# - register (C program)
# - tkmedit (tk application)
# - mri_tessellate (C program)
# - mris_smooth (C program)
# - mris_inflate (C program)
# - mris_sphere
# - mris_register
# - mris_flatten

# SPECIAL COMPILATION
# - mri_regavg (C program)
# - rsh (expect script lives on muir, for remote COCGV registration)
# - RegisterAnatomies.tcl (tcl script lives on /apps/cocgv)
# - analyze2bshort (C program)
# - excise (C, program)
# - analyze2mgh (C program)
# - ge2bshort (C program)
# - ParseVolume (C program)
# - GetAVWInfo (C program)
# - remotetkmedit (expect script for remote operation of tkmedit)




#
#
# Beginning of Procedure definitions
#
#

# This procedure sets paths to executables which
# are called throughout the script
proc SetSurfPaths {} {

        global TCLPLATFORM 
	global TCLPLATFORMOS

	global BINARIES
	global BINARIES2
	global MNIBINARIES
	global MODEL
	global MRI_DIR
	global AFNIBIN
        # variables related to mri_strip_skull and wmfilter
        global FZERO 
        global FMAX
        global DFRAC
        global ISTILT
        global WHITE_HILIM
        global WHITE_LOLIM
        global GRAY_HILIM
	global env

	# determine platform of execution, os type and display
	# this information determines library and path settings
	# for the flattening executables, tcl/tk, etc
	# note the use of tcl_platform requires that the script
	# be executed from tclsh or wish version 8.0 or higher
	
	
	# determine whether execution is from a unix machine
	if { [string compare "unix" $TCLPLATFORM] } {
	    error "SetSurfPaths: you must be running on a unix operating system"
	}

	# determine whether execution is an SGI or a LINUX 

	if { [string compare "IRIX64" $TCLPLATFORMOS] == 0 || [string compare "IRIX" $TCLPLATFORMOS] == 0 } {
	    set platformtype "sgi"
	} elseif { [string compare "Linux" $TCLPLATFORMOS] == 1 } {
	    set platformtype "linux" 
	} else {
	    error "SetSurfPaths: your platform os is $TCLPLATFORMOS, must be IRIX64 or Linux"
	}

	# depending on whether execution is on an SGI or a LINUX system
	# set paths appropriately

	switch -exact -- $platformtype {
	
		
	    sgi {

		# SGI set-up

		# set the version path
		# previous version
		# set VERSION /hippo/apps/flat/recon.11-16-98-IRIX
		# most up-to-date

		# version directory
		# set VERSION /hippo/apps/flat/IRIX/csurf
                 set VERSION /hippo/apps/flat/recon.1-13-99.IRIX/mri 
		if { ![file isdirectory $VERSION] } {
		    error "SetSurfPaths: cannot find version directory, $VERSION"
		}

		

		# location of executables
		# set BINARIES $VERSION/bin/Irix
		 set BINARIES $VERSION/Irix
		#set BINARIES $VERSION/bin/IRIX
		if { ![file isdirectory $BINARIES] } {
		    error "SetSurfPaths: cannot find executables directory, $BINARIES"
		}

	
		# location of ancillary binaries
		set BINARIES2 /hippo/apps/flat/IRIX/bin
		if { ![file isdirectory $BINARIES2] } {
		    error "SetSurfPaths: cannot find ancillary executables directory, $BINARIES2"
		}

		
		# location of MNI binaries
		set MNIBINARIES /usr/local/bin
		if { ![file isdirectory $MNIBINARIES] } {
		    error "SetSurfPaths: cannot find MNI binaries directory, $MNIBINARIES"
		}


		# location of 305 average model brain
		set MODEL /usr/local/lib/model/average_305.mnc
		if { ![file exists $MODEL] } {
		    error "SetSurfPaths: cannot find average 305 MNI brain, $MODEL"
		}

		# this needs to be set so that the automated skull
		# removal program knows where to look for the "bem"
		# directory, the contents of which are absolutely
		# necessary for this step to work correctly
		set MRI_DIR $VERSION
		if { ![file isdirectory $MRI_DIR] } {
		    error "SetSurfPaths: cannot find MRI_DIR, $MRI_DIR"
		}

	
		# set the location of the afni executables
		# afni's to3d is used to make anatomy BRICKs
		# for viewing
		set AFNIBIN /usr/people/cox/abin
		if { ![file isdirectory $AFNIBIN] } {
		    error "SetSurfPaths: cannot find loaction of afni executables AFNIBIN, $AFNIBIN"
		}

		# note: there appears to be a version conflict
		# tksurfer and tkmedit do not like the tcl/tk 8.0.4 libraries
		# so I'm using the 7.4 libs instead

		# locations of the tcl/tk libraries
		#set TCL_LIBRARY /usr/local/lib/tcl8.0 
		set TCL_LIBRARY /usr/sgitcl/lib
		if { ![file isdirectory $TCL_LIBRARY] } {
		    error "SetSurfPaths: cannot find tcl library, $TCL_LIBRARY"
		}

		#set TK_LIBRARY /usr/local/lib/tk8.0
		set TK_LIBRARY /usr/sgitcl/lib/tk
		if { ![file isdirectory $TK_LIBRARY] } {
		    error "SetSurfPaths: cannot find tk library, $TK_LIBRARY"
		}

		set env(TCL_LIBRARY) $TCL_LIBRARY
		set env(TK_LIBRARY) $TK_LIBRARY
		set env(MRI_DIR) $MRI_DIR


	    }

	    linux {

		# Linux set-up

		error "SetSurfPaths: still working on Linux configuration"


	    }


	}
	
	# set up platform-independent default environment variables
        
        # for diagnostic purposes
	set env(DIAG) 0x04048
	
        # set default parameters for mri_strip_skull and wmfilter
        # allow the user to change these in the utilities menu
        
	# standard parameters for fast spgr
        set env(FZERO) 50
        set env(FMAX) 120
        set env(DFRAC) 0.7
        set env(ISTILT) 1.0
        set env(WHITE_HILIM) 130
        set env(WHITE_LOLIM) 90
        set env(GRAY_HILIM) 100

	# for 2NEX 
	#set env(FZERO) 20
	#set env(FMAX) 130
	#set env(DFRAC) 0.65
	#set env(ISTILT) 2.0
	#set env(WHITE_HILIM) 130 
	#set env(WHITE_LOLIM) 75 
	#set env(GRAY_HILIM) 85
        

	puts stdout {******************************************************************}
	puts stdout {------------------------------------------------------------------}
	puts stdout {**************  MakeSurf Environment Configuration  **************}
	puts stdout ""
	puts stdout "BINARIES = $BINARIES"
	puts stdout "BINARIES2 = $BINARIES2"
	puts stdout "MNIBINARIES = $MNIBINARIES"
	puts stdout "MODEL = $MODEL"
	puts stdout "MRI_DIR = $MRI_DIR"
	puts stdout "AFNIBIN = $AFNIBIN"
	puts stdout "TCL_LIBRARY = $env(TCL_LIBRARY)"
	puts stdout "TK_LIBRARY = $env(TK_LIBRARY)"
	

	# pass back 0!
	return 0
	
}

# This procedure scrolls down N lines
# - example call: ScrollN 10

proc ScrollN { Num_Scrolls } {

    	set i 0
    	while {$i < $Num_Scrolls} {
        	puts stdout ""
        	incr i
    	}
}




# set Ifiles [glob rep3.*]
# set i 1
# set t 1
# foreach I $Ifiles {
#     set cur_I [format %s%03d rep3. $i]	
#     set new_name [format %s%03d I. $t]
#     puts "Moving $cur_I to $new_name . . ."
#     exec mv $cur_I $new_name 	
#     incr i 1
#     incr t 1
# }


# renames I files that begin with a number 
# greater than 1
proc RenameIfiles { StartNum } {

    set Ifiles [glob I.*]

    set i $StartNum
    set t 1

    foreach I $Ifiles {

	set cur_I [format %s%03d I. $i]
	
	set new_name [format %s%03d I. $t]

	puts "Moving $cur_I to $new_name . . ."
	exec mv $cur_I $new_name 
	
	incr i 1
	incr t 1
    }
}

# this proc converts degrees to radians
proc Deg2Rad { Angle } {

    # rad = pi/180*angle
    set Radian [expr (6*abs(asin(0.5))/180)*$Angle] 

    return $Radian
}

# this proc converts radians to degrees
proc Rad2Deg { Radian } {

   # angle = 180*radian/pi
    set Angle [expr (180*$Radian)/(6*abs(asin(0.5)))]

    return $Angle

}


# this procedure creates the 4-by-4 homogenous transformation matrix which
# the program register needs to run correctly
proc MakeRegisterMatrix { regfname xvoxsize zvoxsize} {

    global subjname

    # its safe to open the register.dat file for writing
    if [catch {open $regfname w} register_dat_fid] {
	error "MakeRegisterMatrix: Can't open for writing: $funcdir/register.dat"
    }

    # fill the register.dat file with information
    # the format of the register.dat file is:
    # subjname
    # x voxel size (mm)
    # z voxel size (mm)
    # a scale factor (?) which may have something to do with image display brightness 
    # the 4-by-4 transformation matrix

    # ask questions about translations
    
    puts stdout "Parameters for 4-by-4 transformation matrix initialization" 
    puts stdout ""
    puts stdout "x-axis shift (mm)"
    puts -nonewline stdout {: }
    flush stdout  ;#
    set xshift [gets stdin]

    puts stdout ""
    puts stdout "y-axis shift (mm)"
    puts -nonewline stdout {: }
    flush stdout  ;#
    set yshift [gets stdin]

    puts stdout ""
    puts stdout "z-axis shift (mm)"
    puts -nonewline stdout {: }
    flush stdout  ;#
    set zshift [gets stdin]

    # ask questions about rotations
    puts stdout ""
    puts stdout "pitch, x-axis rotation (degrees)"
    puts -nonewline stdout {: }
    flush stdout  ;#
    set theta [gets stdin]
    # convert degrees to radians
    set theta [Deg2Rad $theta]

    puts stdout ""
    puts stdout "roll, y-axis rotation (degrees)"
    puts -nonewline stdout {: }
    flush stdout  ;#
    set phi [gets stdin]
    # convert degrees to radians
    set phi [Deg2Rad $phi]
    
    puts stdout ""
    puts stdout "yaw, z-axis rotation (degrees)"
    puts -nonewline stdout {: }
    flush stdout  ;#
    set psi [gets stdin]
    # convert degrees to radians
    set psi [Deg2Rad $psi]

    # compute each element of the 4-by-4 homogeneous transformation matrix
    # the variables correspond to the matrix elements by the following convention

    #    a    b    c    d
    #    e    f    g    h
    #    i    j    k    m
    #    n    p    q    1    

		    
    set h [format %13.6e $yshift]
    set m [format %13.6e $zshift]

    # compute a as:   (cos(psi)*cos(phi)) + (sin(phi)*sin(theta)*sin(phi))
    set a [format %13.6e [expr cos($psi) * cos($phi) + sin($phi) * sin($theta) * sin($phi)]]
		    
    # compute b as:   (sin(psi)*cos(phi)) - (cos(psi)*sin(theta)*sin(phi))
    set b [format %13.6e [ expr sin($psi) * cos($phi) - cos($psi) * sin($theta) * sin($phi)]]

    # compute c as:   cos(theta)*sin(phi)
    set c [format %13.6e [expr cos($theta) * sin($phi)]]

    # compute d as:   shift along x-axis
    set d [format %13.6e $xshift]

    # compute e as:   -sin(psi)*cos(theta)  
    set e [format %13.6e [expr -1*sin($psi) * cos($theta)]]

    # compute f as:   cos(psi)*cos(theta)
    set f [format %13.6e [expr cos($psi) * cos($theta)]]

    # compute g as:   sin(theta)
    set g [format %13.6e [expr sin($theta)]]

    # compute h as:   shift along y-axis
    set h [format %13.6e $yshift]

    # compute i as:   (sin(psi)*sin(theta)*cos(phi) - cos(psi)*sin(phi))
    set i [format %13.6e [ expr sin($psi) * sin($theta) * cos($phi) - cos($psi) * sin($phi) ]]

    # compute j as:   (-cos(psi)*sin(theta)*cos(phi) - sin(psi)*sin(theta))
    set j [format %13.6e [ expr -1*cos($psi) * sin($theta) * cos($phi) - sin($psi) * sin($theta) ]]

    # compute k as:   cos(theta)*cos(phi)
    set k [format %13.6e [expr cos($theta) * cos($phi)]]

    # compute m as:   shift along z-axis
    set m [format %13.6e $zshift]

    # compute n as: 1/xview, or the x coordinate from which the image is viewed
    set n [format %13.6e 0]

    # compute p as: 1/yview, or the y coordinate from which the image is viewed
    set p [format %13.6e 0]

    # compute q as: 1/zview, or the z coordinate from which the image is viewed
    set q [format %13.6e 0]

    set r [format %13.6e 1]
		    
    # initialize transformation matrix
    puts stdout ""
    puts stdout "4-by-4 Homogeneous Transformation Matrix:"
    puts stdout ""
    puts stdout " $a $b $c $d"
    puts stdout " $e $f $g $h"
    puts stdout " $i $j $k $m"
    puts stdout " $n $p $q $r"
    puts stdout ""
		    
		    
    puts $register_dat_fid "$subjname"
    puts $register_dat_fid "$xvoxsize"
    puts $register_dat_fid "$zvoxsize"
    puts $register_dat_fid "0.50"
		    
    puts $register_dat_fid " $a $b $c $d"
    puts $register_dat_fid " $e $f $g $h"
    puts $register_dat_fid " $i $j $k $m"
    puts $register_dat_fid " $n $p $q $r"
    close $register_dat_fid

    return 0


}

# this procedure reads and displays the contents of an analyse.dat file
proc ReadAnalyseDat { fname } {

     # open analyse.dat file
    if [catch {open $fname r} channel] {
	error "ReadAnalyseDat: can't open analyse.dat file"
    }

    
    set numlines 1
    while {[gets $channel line] >= 0} {

	if { $numlines == 1 } {
	    puts stdout "Registration directory: $line"
	}
	if { $numlines == 2 } {
	     puts stdout "Registration Image name: $line"
	}
	if { $numlines == 3 } {
	     puts stdout "Z Dimension, Number Timepoints: $line"
	}
	if { $numlines == 4 } {
	     puts stdout "X and Y Dimensions: $line"
	}
	
	incr numlines
    }

    # close analyse.dat file
    close $channel

    return 0

}

# this procedure reads and displays the contents of an register.dat file
proc ReadRegisterDat { fname } {

     # open register.dat file
    if [catch {open $fname r} channel] {
	error "ReadRegisterDat: can't open register.dat file"
    }

    
    set numlines 1
    while {[gets $channel line] >= 0} {

	if { $numlines == 1 } {
	    puts stdout "Surface directory: $line"
	}
	if { $numlines == 2 } {
	     puts stdout "X, Y Voxel Size (mm): $line"
	}
	if { $numlines == 3 } {
	     puts stdout "Z Voxel Size (mm): $line"
	}
	if { $numlines == 4 } {
	     puts stdout "Brightness factor: $line"
	     puts stdout "Transformation matrix:"
	}
	if { $numlines >= 5 && $numlines <= 8 } {
	    puts $line
	}
	incr numlines
    }

    # close register.dat file
    close $channel

    return 0

}


# Reads AFNI HEAD file to get image information
# Takes pathname of the AFNI HEAD file and
# returns a list containing 6 parameters:
# xdim 
# ydim 
# zdim 
# xvoxdim 
# yvoxdim 
# zvoxdim

proc ReadAFNIHEAD { HEADName } {

    # open HEAD file
    if [catch {open $HEADName r} channel] {
	error "ReadAFNIHEAD: can't open HEAD file"
    }

    set numlines 1
    while {[gets $channel line] >= 0} {

	# check for the title line that occurs TWO lines before the voxel dimensions are listed
	set voxdimtitle "name  = DELTA"
	if {![string compare $voxdimtitle $line]} {
	    set voxdimtitle_num $numlines
	    # then the voxel sizes will occur on this line
	    set voxdimlinenum [expr $voxdimtitle_num +2]
	}

	# check to see if voxeldimlinenum variable has been set yet
	# if it has then read that line and extract the voxel dimensions
	if { [info exists voxdimlinenum] && $numlines == $voxdimlinenum } {
	    set voxdimline $line
	}

	# check for the title line that occurs TWO lines before the image dimensions are listed
	set imgdimtitle "name = DATASET_DIMENSIONS"
	if {![string compare $imgdimtitle $line]} {
	    set imgdimtitle_num $numlines
	    # then the image dimension sizes will occur on this line
	    set imgdimlinenum [expr $imgdimtitle_num +2]
	}

	# check to see if imgdimlinenum variable has been set yet
	# if it has then read that line and extract the image dimensions
	if { [info exists imgdimlinenum] && $numlines == $imgdimlinenum } {
	    set imgdimline $line 
	}
	
	incr numlines
    }

    #puts stdout ""
    #puts stdout "Number of lines: $numlines"
    #puts stdout "Voxel dims title line occured at line: $voxdimtitle_num, data are on line $voxdimlinenum"
    #puts stdout "Image dims title line occured at line: $imgdimtitle_num, data are on line $imgdimlinenum"
    #puts stdout ""
    #puts stdout "Voxel dimensions: $voxdimline"
    #puts stdout "Image dimensions: $imgdimline"
    # scan in the relevant voxel dimensions 
    set num_vox_dim [scan $voxdimline {%6f %6f %6f} xvoxdim yvoxdim zvoxdim]
    if { $num_vox_dim != 3 } {
	error "ReadAFNIHEAD: did not find valid voxel sizes for x, y, and z dimensions"
    }
    #puts stdout "X Voxel Dim: $xvoxdim"
    #puts stdout "Y Voxel Dim: $yvoxdim"
    #puts stdout "Z Voxel Dim: $zvoxdim"
    # scan in the relevant image dimensions 
    set num_img_dim [scan $imgdimline {%6d %6d %6d} xdim ydim zdim]
    if { $num_img_dim != 3 } {
	error "ReadAFNIHEAD: did not find valid sizes for x, y, and z dimensions"
    }
    #puts stdout "X Image Dim: $xdim"
    #puts stdout "Y Image Dim: $ydim"
    #puts stdout "Z Image Dim: $zdim"

    # make the header information list
    set headerinfo [list $xdim $ydim $zdim $xvoxdim $yvoxdim $zvoxdim]

    # close HEAD file
    close $channel

    return $headerinfo

}


proc MakeDirHierarchy { } {

	# make the directory hierarchy
	
	exec mkdir -p mri/T1
	exec mkdir -p mri/bem
	exec mkdir -p mri/brain
	exec mkdir -p mri/filled 
	exec mkdir -p mri/orig
	exec mkdir -p mri/tmp
	exec mkdir -p mri/transforms
	exec mkdir -p mri/wm
	exec mkdir -p mri/stat
	exec mkdir -p mri/epi
	exec mkdir -p mri/stat2T1
	exec mkdir -p mri/stat2epi

	# the bem directory needs to be on the same level
	# as the mri directory; bem holds the brain.tri
	# file output by mri_strip_skull
	exec mkdir bem
	
	# the surf directory holds the reconstructed
	# surfaces for the right and left hemispheres
	exec mkdir surf
	
	# the func directory holds the functional 
	# statistical maps which will be used for
	# overlay
	exec mkdir func

	#
	# the anfni directory holds anatomical
	# BRICKs that will be created to visualize
	# each stage of the surface processing
	exec mkdir afni

	#make a tmp directory at the upper level
	exec mkdir tmp

	exec mkdir struct
	exec mkdir scripts
	exec mkdir rgb
	exec mkdir label
	exec mkdir raw

}


proc PreProcessSurf { } {

	global processdir
	global subjname
	global slicethickness
	global FOV
	global SUBJECTS_DIR
	global Ifiledir
	global NumSlices
	global BINARIES
	global BINARIES2
	global AFNIBIN
	global env

	
	#
	# Get user input
	#
	

	puts stdout ""
	puts stdout {Enter a directory path (absolute or relative) that}
	puts stdout {will be used to store the results of the surface}
	puts stdout {based analysis}
	puts -nonewline stdout {:  }
	flush stdout  ;#
	set processdir [gets stdin]


	# test to see whether directory already exists
	if {[file isdirectory $processdir] == 1} {
    	   error "The directory you specified, $processdir, already exists."
	}

	# now make the directory

	exec mkdir -p $processdir
	
	# open the log file for reading and writing whether or not it exists 

	if [catch {open $processdir/MakeSurf.log a+ 0666} logfileId] {
	    error "PreProcessSurf - Cannot open $processdir/MakeSurf.log: $logfileId"
	}
 
	cd $processdir

	puts $logfileId {******************************************************************}
	set Date [exec date]

	puts $logfileId "$Date  :  $env(USER)  :  MakeSurf"
	puts $logfileId "MakeSurf: Automated Surface Analysis Script Logfile"

	puts $logfileId ""
	puts $logfileId "Directory containing the surface-based processing hierarchy: [exec pwd]"
	
	set processdir [exec pwd]

	# cd to the processing directory 

	cd $processdir

	# get the tail of the processing directory
	# in Dale-speak this is the subjname variable

	set subjname [file tail [exec pwd]]

	# get the directory path up to but not including
	# the last name. In Dale-speak this the SUBJECTS_DIR
	# variable which will be used as an environment variable

	set SUBJECTS_DIR [file dirname [exec pwd]]
	
	#make the directory hierarchy
	
	MakeDirHierarchy
	
	# put I file processing here

	# ask for number of sets of I files to be used in processing
	puts stdout ""
	puts stdout "You have the option of choosing one or more anatomicals to be"
	puts stdout "use in creating the initial surface reconstruction. If you"
	puts stdout "choose more than 1 then the anatomicals will be registered"
	puts stdout "and and then averaged."
	puts stdout ""
	puts -nonewline stdout "Enter number of anatomicals to be used in creating the surface: "
	flush stdout  ;#
	set numanatomies [gets stdin]

	# Check for valid number of P-files
	if {$numanatomies < 1} {
	    error "$numanatomies is not a valid number!"
	}


	puts $logfileId ""
	puts $logfileId "Number of I files to be used in the surface creation: $numanatomies"

	# if there is only one set of I files to be used in the surface creation
	if { $numanatomies == 1 } {
	
	    puts stdout ""
	    puts -nonewline stdout "Enter the directory where your I files are located: "
	    flush stdout  ;#
	    set Ifiledir [gets stdin]

	    puts $logfileId ""
	    puts $logfileId "Directory containing the I files: $Ifiledir"

	    puts stdout ""
	    puts -nonewline stdout "Enter the slice thickness in mm: "
	    flush stdout  ;#
	    set slicethickness [gets stdin]

	    puts $logfileId ""
	    puts $logfileId "Slice thickness (mm): $slicethickness"

	    puts stdout ""
	    puts -nonewline stdout "Enter the field of view (FOV) in mm: "
	    flush stdout  ;#
	    set FOV [gets stdin]

	    puts $logfileId ""
	    puts $logfileId "FOV (mm): $FOV"

	    # check to see whether these are signa 5X I files or LX IMA files
	    
	    if { [llength [glob -nocomplain $Ifiledir/I.*]] } {
		# signa 5X
		set Ifiles [glob $Ifiledir/I.*]
		set Ifiletype "signa5X"
	    } elseif { [llength [glob $Ifiledir/*.IMA]] } {
		# signa LX
		set Ifiles [glob -nocomplain $Ifiledir/*.IMA]
		set Ifiletype "signaLX"
	    } else {
		error "Cannot determine raw anatomical file type"
	    }

	   
	    # add a check to see if the I file numbers begin at I.001.
	    # if not the Dale processing sofware will fail
	
	    # the number of I files is equal to the
	    # number of slices in the anatomical volume

	    # get the start time for processing
	    set StartTime [exec date]

	    # to the screen . . .
	    puts stdout ""
	    puts stdout {Linking to raw MR files and creating bshort sliceplanes .  .  .}
	    puts stdout ""
	    puts stdout ". . . starting at $StartTime . . ."

	    # to the log file
	    puts $logfileId ""
	    puts $logfileId "Linking to raw MR files and creating bshort sliceplanes .  .  ."
	    puts $logfileId ""
	    puts $logfileId ". . . starting at $StartTime . . ."

	    set i 0
	    foreach I $Ifiles {
		set cur_I [lindex $Ifiles $i]
		set cur_I_tail [file tail $cur_I]
		# make symbolic links
		exec ln -s $cur_I $SUBJECTS_DIR/$subjname/mri/orig/$cur_I_tail 
		
		incr i 1
	    }
	
	    # first go to the location of the I file links
	    cd $SUBJECTS_DIR/$subjname/mri/orig

	    switch -exact -- $Ifiletype {
	
		"signa5X" {	
		    set ILinks [glob I.*]
		}

		"signaLX" {
		    set ILinks [glob *.IMA]
		}

	    }
	
	    # Convert ge raw files to the the MGH bshort format
	    # if signa 5X use the program ge2bshort written by folks at MGH
	    # if signa LX use the program excise to strip header and make bshort files
	    set i 0
	
	    foreach I $ILinks {	

		set cur_I [lindex $ILinks $i]
		
		switch -exact -- $Ifiletype {
	
		    "signa5X" {
			
			puts stdout "Processing signa 5X file: $cur_I"

			# call ge2bshort in quiet mode
			set ge2bshort_result [catch {exec $BINARIES2/ge2bshort -q $cur_I} ge2bshort_msg]
			

		    }

		    "signaLX" {
			
			puts stdout "Processing signa LX file: $cur_I"
			# create bshort image file
			set excise_result [catch {exec $BINARIES2/excise $cur_I [format %s%03d%s "EI." [expr $i + 1] "S00001.bshort"] 9376 131072 >&@ stdout} excise_msg]
			# create corresponding hdr file
			set bfloathdr [format %s%03d%s "EI." [expr $i + 1] "S00001.hdr"]
			# open hdr file
			if [catch {open $bfloathdr w} bfloathdr_fid] {
			    error "PreProcessSurf: Can't open for writing: $bfloathdr"
			}
			# fill with bfloat image plane information
			puts $bfloathdr_fid "256 256 1 0"
			# close hdr file
			close $bfloathdr_fid 
		
		    }

		}
			
		# write program output to the logfile
		# puts $logfileId $ge2bshort_msg
		# get rid of the symbolic link
		# exec rm $cur_I
		incr i 1
		# get rid of the symbolic link
		exec rm $cur_I
	    }
	
	    set NumSlices $i

	    # create the COR files
	    CreateCorFiles

	} elseif { $numanatomies > 1} {

	    # accumulate information for each anatomy

	    set anatomyindex 1
	    while {$anatomyindex <= $numanatomies} {

		puts stdout ""
		puts stdout "**********************************************"
		puts stdout "Information for Anatomy Number: $anatomyindex "
		puts $logfileId ""
		puts $logfileId "**********************************************"
		puts $logfileId "Information for Anatomy Number: $anatomyindex "
		
		puts -nonewline stdout "Enter the directory where your I files are located: "
		flush stdout  ;#
		set Ifiledir [gets stdin]

		lappend Ifiledirlist $Ifiledir

		puts $logfileId ""
		puts $logfileId "Directory containing the I files: $Ifiledir"

		puts stdout ""
		puts -nonewline stdout "Enter the slice thickness in mm: "
		flush stdout  ;#
		set slicethickness [gets stdin]

		lappend slicethicknesslist $slicethickness

		puts $logfileId ""
		puts $logfileId "Slice thickness (mm): $slicethickness"

		puts stdout ""
		puts -nonewline stdout "Enter the field of view (FOV) in mm: "
		flush stdout  ;#
		set FOV [gets stdin]

		lappend FOVlist $FOV

		puts $logfileId ""
		puts $logfileId "FOV (mm): $FOV"

		incr anatomyindex
	    }

	    # for each anatomy the I file dir, FOV and slicethickness are stored in lists

	    # now process each set of I files 
	    # the result will be bias corrected afni BRIKs for each anatomical 
	    # volume stored in $SUBJECTS_DIR/$subjname/afni

	   

	    set anatomyindex 0
	    foreach Ifiledir $Ifiledirlist {

		set slicethickness [lindex $slicethicknesslist $anatomyindex]
		set FOV [lindex $FOVlist $anatomyindex]

		# check to see whether these are signa 5X I files or LX IMA files
	    
		if { [llength [glob -nocomplain $Ifiledir/I.*]] } {
		    # signa 5X
		    set Ifiles [glob $Ifiledir/I.*]
		    set Ifiletype "signa5X"
		} elseif { [llength [glob -nocomplain $Ifiledir/*.IMA]] } {
		    # signa LX
		    set Ifiles [glob -nocomplain $Ifiledir/*.IMA]
		    set Ifiletype "signaLX"
		} else {
		    error "Cannot determine raw anatomical file type"
		}

		
		# get the start time for processing
		set StartTime [exec date]

		# to the screen . . .
		puts stdout ""
		puts stdout "Linking to raw MR files and creating bshort sliceplanes for anatomy number [expr $anatomyindex + 1] .  .  ."
		puts stdout ""
		puts stdout ". . . starting at $StartTime . . ."

		# to the log file
		puts $logfileId ""
		puts $logfileId "Linking to raw MR files and creating bshort sliceplanes for anatomy number [expr $anatomyindex + 1] .  .  ."
		puts $logfileId ""
		puts $logfileId ". . . starting at $StartTime . . ."

		# before making the I file links check to see that there are no old links there
		# first go to the location of the I file links
		cd $SUBJECTS_DIR/$subjname/mri/orig
		
		set i 0
		foreach I $Ifiles {
		    set cur_I [lindex $Ifiles $i]
		    set cur_I_tail [file tail $cur_I]
		    # make symbolic links
		    # puts stdout "Linking $cur_I to [format %s/%s/mri/orig/%s $SUBJECTS_DIR $subjname $cur_I_tail]"
		    exec ln -s $cur_I $SUBJECTS_DIR/$subjname/mri/orig/$cur_I_tail 
		    
		    incr i 1
		}
	
		switch -exact -- $Ifiletype {
	
		    "signa5X" {	
			set ILinks [glob I.*]
		    }

		    "signaLX" {
			set ILinks [glob *.IMA]
		    }

		}
	
		
		set i 0
		foreach I $ILinks {	

		    set cur_I [lindex $ILinks $i]

		    switch -exact -- $Ifiletype {
	
			"signa5X" {
			
			    puts stdout "Processing signa 5X file: $cur_I"

			    # call ge2bshort in quiet mode
			    set ge2bshort_result [catch {exec $BINARIES2/ge2bshort -q $cur_I} ge2bshort_msg]
			    

			}

			"signaLX" {

			    puts stdout "Processing signa LX file: $cur_I"
			    set excise_result [catch {exec $BINARIES2/excise $cur_I [format %s%03d%s "EI." [expr $i + 1] "S00001.bshort"] 9376 131072 >&@ stdout} excise_msg]
			    # create corresponding hdr file
			    set bfloathdr [format %s%03d%s "EI." [expr $i + 1] "S00001.hdr"]
			    # open hdr file
			    if [catch {open $bfloathdr w} bfloathdr_fid] {
				error "PreProcessSurf: Can't open for writing: $bfloathdr"
			    }
			    # fill with bfloat image plane information
			    puts $bfloathdr_fid "256 256 1 0"
			    # close hdr file
			    close $bfloathdr_fid 

			}

		    }
		    
		    # get rid of the symbolic link
		    exec rm $cur_I
		    incr i 1
		}
		
		set NumSlices $i
		
		# make BRIK and histogram names for raw data
		set raw_brikname [format raw_%d_of_%d [expr $anatomyindex + 1] $numanatomies]
		set raw_histname [format raw_%d_of_%d_hist.txt [expr $anatomyindex + 1] $numanatomies]

		# accumulate list of raw names
		lappend raw_briknames_list $raw_brikname
		lappend raw_histnames_list $raw_histname

		# create raw COR files, afni BRIK, and histogram
		CreateCorFiles $raw_brikname $raw_histname

		# make BRIK and histogram names for intensity normalized data
		set norm_brikname [format norm_%d_of_%d [expr $anatomyindex + 1] $numanatomies]
		set norm_histname [format norm_%d_of_%d_hist.txt [expr $anatomyindex + 1] $numanatomies]

		# accumulate list of normalized names
		lappend norm_briknames_list $norm_brikname
		lappend norm_histnames_list $norm_histname

		# create intensity normalized COR files, afni BRIK, and histogram
		CorrectBiasField $norm_brikname $norm_histname

		incr anatomyindex

	    }

	    # first go to the location of the unregistered raw and intensity normalized BRIKS
	    # the names of the BRIKs are stored as follows:
	    # norm_%d_of_%d, raw_%d_of_%d

	    cd $SUBJECTS_DIR/$subjname/afni

	    # set the standard file for registration
	    # this will be the first intensity normalized BRIK

	    set standardfile [format %s/%s/afni/%s+orig.BRIK $SUBJECTS_DIR $subjname [lindex $norm_briknames_list 0]]
	    

	    # check to see if standard file exists
	    if { [file exists $standardfile] == 0 } {
		error "Standard file, $standardfile,  does not exist!"
	    }

	    # starting with the SECOND intensity normalized BRIK, register each to the standard file
	    set normanatomyindex 1
	    while { $normanatomyindex < [llength $norm_briknames_list] } {
		
		# set the reslice file name 
		set reslicefile [format %s/%s/afni/%s+orig.BRIK $SUBJECTS_DIR $subjname [lindex $norm_briknames_list $normanatomyindex]]

		# check to see if reslice file exists
		if { [file exists $reslicefile] == 0 } {
		    error "Reslice file, $reslicefile, does not exist!"
		}

		# do the registration by calling "rsh" an expect script which
		# spawns an rlogin session on kampos.nimh.nih.gov where the
		# COCGV registration and reslice code are used

		puts stdout ""
		puts stdout "Registering: $reslicefile"
		puts stdout "to . . . "
		puts stdout "$standardfile at: [exec date]"
		puts stdout ""
	
		puts $logfileId ""
		puts $logfileId "Registering: $reslicefile"
		puts $logfileId "to . . . "
		puts $logfileId "$standardfile at: [exec date]"
		puts $logfileId ""

		set remotereg_result [catch {exec $BINARIES2/rsh kampos /apps/cocgv/RegisterAnatomies.tcl $standardfile $reslicefile >&@ stdout} remotereg_msg]

		puts stdout ""
		puts stdout "Finished Registering: $reslicefile"
		puts stdout "to . . . "
		puts stdout "$standardfile at: [exec date]"
		puts stdout ""

		puts $logfileId ""
		puts $logfileId "Finished Registering: $reslicefile"
		puts $logfileId "to . . . "
		puts $logfileId "$standardfile at: [exec date]"
		puts $logfileId ""

		# alternatively register using afni's 3dvolreg
		# set outprefix [format %s/%s/afni/R%s $SUBJECTS_DIR $subjname [lindex $norm_briknames_list $normanatomyindex]]
		# set dfilename [format %s%s [file rootname $reslicefile] .motioninfo]
		# set basename [format %s+orig%s%s%s.BRIK  [string range $standardfile 0 [expr [string length $standardfile] - 11]] {[} 0 {]} ]
		# puts stdout "exec 3dvolreg -verbose -verbose -Fourier -clipit -dfile $dfilename -prefix $outprefix -base $basename $reslicefile"
                # exec 3dvolreg -verbose -verbose -Fourier -clipit -dfile $dfilename -prefix $outprefix -base $basename $reslicefile >&@ stdout
		
		# store the name of the output registered intensity normalized volume in a list
		# the naming convention is to prepend a captial R before the name
		set outputfile [format %s/%s/afni/R%s+orig.BRIK $SUBJECTS_DIR $subjname [lindex $norm_briknames_list $normanatomyindex]]

		lappend registeredvols_list $outputfile

		incr normanatomyindex
	    }

	  # append the standard volume to the list of registered volumes to create the final list for averaging

	  lappend registeredvols_list $standardfile

	  # do the averaging using mri_regavg 
	  # it places COR files in $SUBJECT_DIR/$subjname/mri/orig

	  puts stdout ""
	  puts stdout "Averaging Registered Volumes:"
	    foreach volume $registeredvols_list {
		puts stdout "$volume"
	    }
	  puts stdout ""

	  puts $logfileId ""
	  puts $logfileId "Averaging Registered Volumes:"
	    foreach volume $registeredvols_list {
		puts stdout "$volume"
	    }
	  puts $logfileId ""

	  set cmd "exec $BINARIES2/mri_regavg [format %s $registeredvols_list] [format %s/%s/mri/orig $SUBJECTS_DIR $subjname] >&@ stdout"
	    
	  set mri_regavg_result [catch { eval $cmd } mri_regavg_result]
	   
	}
	
	# get the end time for processing
	set EndTime [exec date]

	puts stdout ". . . ending at $EndTime . . ."
	puts $logfileId ". . . ending at $EndTime . . ."
	puts ""
	
	close $logfileId

	return 0
}

# DisplayHierachy
# - this procedure displays the user input
# before starting the surface creation step
#
#

proc DisplayHierarchy { } {

	global SUBJECTS_DIR
	global $subjname

	set processdir [concat $SUBJECTS_DIR/$subjname]

	if [catch {open $processdir/MakeSurf.log a+ 0666} logfileId] {
	    puts stderr "DisplayHierarchy - Cannot open $processdir/MakeSurf.log: $logfileId"
	}

	# to the screen . . .
	puts stdout ""
	puts stdout {******************************************************************}
	puts stdout {****          BEGINNING SURFACE CREATION PROCESSING           ****}
	puts stdout ""

	# to the log file
	puts $logfileId ""
	puts $logfileId {******************************************************************}
	puts $logfileId {****          BEGINNING SURFACE CREATION PROCESSING           ****}
	puts $logfileId ""

	close $logfileId
	
}




#
# CreateCorFiles -
# creation of 1 cubic mm coronal 8-bit images
#
# this proc calls: ge2bshort, bshortconv, mri_convert
#

proc CreateCorFiles { {BRIKNAME raw} {HISTNAME rawhist.txt} } {

	global SUBJECTS_DIR
	global subjname
	global NumSlices
	global slicethickness
	global FOV
	global BINARIES
	global BINARIES2
	global VERBOSE_FLAG
	global AFNIBIN
	global env

	set processdir [concat $SUBJECTS_DIR/$subjname]

	if [catch {open $processdir/MakeSurf.log a+ 0666} logfileId] {
	    error "Cannot open $processdir/MakeSurf.log: $logfileId"
	}
	
	# get the start time for processing
	set StartTime [exec date]

	puts stdout ""
	puts stdout {Creating 1.0 cubic mm coronal 8-bits/pixel images .  .  .}
	puts stdout ""
	puts stdout ". . . starting at $StartTime . . ."
	puts $logfileId ""
	puts $logfileId "Creating 1.0 cubic mm coronal 8-bit images .  .  ."
	puts $logfileId ""
	puts $logfileId ". . . starting at $StartTime . . ."
	
	# go up to top level (at mri directory) 
	# there used to be a "scripts" directory here

	cd $SUBJECTS_DIR/$subjname

	# bshortconv needs an environment variable set

	set env(SUBJECTS_DIR) $SUBJECTS_DIR

	# do the isotransform and datatype conversion with bshortconv

	if {$VERBOSE_FLAG} {
	 set bshortconv_result [catch {exec $BINARIES2/bshortconv $subjname EI.%03dS00001.bshort \
	 1 $NumSlices 1 $slicethickness $FOV >&@ stdout} bshortconv_msg]
	} else {
	 set bshortconv_result [catch {exec $BINARIES2/bshortconv $subjname EI.%03dS00001.bshort \
	 1 $NumSlices 1 $slicethickness $FOV >&@ $logfileId} bshortconv_msg]
	}
	


	# get rid of the 16-bit bshort files now that the 8-bit COR files have been made
	if {$VERBOSE_FLAG} {
	 puts stdout ""
	 puts stdout "Removing temporary bshort files .  .  ."
	 puts stdout ""
	}

	set bshortjunk [glob -nocomplain $SUBJECTS_DIR/$subjname/mri/orig/EI.*]
	if { [llength $bshortjunk] == 0 } {
	    error "Cannot find temporary bshort files"
	} else {

	    foreach bfile $bshortjunk {
		exec rm -rf $bfile
	    }
	}

	cd $SUBJECTS_DIR/$subjname/mri

	# Flip in X to preserve radiological orientation: L=R in tkmedit transverse view
	# This ensures that the COR files may be viewed properly in tkmedit
	# and that the right and left surfaces files really are from the right and left
	# sides of the brain. This call assumes that the original I files
	# are sagittal and start from the left-most and end at the right-most
	if {$VERBOSE_FLAG} {
	    set mriconvert_result [catch {exec $BINARIES/mri_convert -x -1 orig orig  >&@ stdout} mriconvert_msg]
	} else {
	     set mriconvert_result [catch {exec $BINARIES/mri_convert -x -1 orig orig  >&@ $logfileId} mriconvert_msg]
	}

	# create afni BRIK for raw MR data

	cd $SUBJECTS_DIR/$subjname

	if {$VERBOSE_FLAG} {
	 puts stdout ""
         puts stdout "Creating AFNI BRIK, $BRIKNAME, and histogram, $HISTNAME, for raw MR data . . ."
         puts stdout ""

         set raw_result [catch {exec $AFNIBIN/to3d -spgr -prefix $BRIKNAME -session $SUBJECTS_DIR/$subjname/afni \
         -view orig -datum byte -orient LSP $SUBJECTS_DIR/$subjname/mri/orig/COR-\[0-256]\* >&@ stdout} raw_msg]

	 set hist_result [catch {exec $AFNIBIN/3dhistog -nbin 255 -notit \
	 $SUBJECTS_DIR/$subjname/afni/$BRIKNAME+orig.BRIK >& $SUBJECTS_DIR/$subjname/afni/$HISTNAME} hist_msg]

	} else {
	 puts $logfileId ""
         puts $logfileId "Creating AFNI BRIK, $BRIKNAME, and histogram, $HISTNAME, for raw MR data . . ."
         puts $logfileId ""

	 set raw_result [catch {exec $AFNIBIN/to3d -spgr -prefix $BRIKNAME -session $SUBJECTS_DIR/$subjname/afni -view orig \
         -datum byte -orient LSP $SUBJECTS_DIR/$subjname/mri/orig/COR-\[0-256]\* >&@ $logfileId} raw_msg]

         set hist_result [catch {exec $AFNIBIN/3dhistog -nbin 255 -notit \
         $SUBJECTS_DIR/$subjname/afni/$BRIKNAME+orig.BRIK >& $SUBJECTS_DIR/$subjname/afni/$HISTNAME} hist_msg]
	}
	
	# get the end time for processing
	set EndTime [exec date]

	puts stdout ". . . ending at $EndTime . . ."
	puts $logfileId ". . . ending at $EndTime . . ."
	puts ""
	
	close $logfileId
	
}


#
# MNINormalizeAnatomy -
# MNI spatial normalization to generate a 12 
# parameter Talairach transformation matrix
#
# this proc calls: mri_convert, mritotal, mri_add_xform_to_header
#

proc MNINormalizeAnatomy { } {

	global SUBJECTS_DIR
	global subjname
	global BINARIES
	global BINARIES2
	global MNIBINARIES
	global VERBOSE_FLAG
	global AFNIBIN
	global env

	set processdir [concat $SUBJECTS_DIR/$subjname]

	if [catch {open $processdir/MakeSurf.log a+ 0666} logfileId] {
	    error "MNINormalizeAnatomy - Cannot open $processdir/MakeSurf.log: $logfileId"
	}
	
	
	# get the start time for processing
	set StartTime [exec date]

	puts stdout ""
	puts stdout {Computing Talairach Transformation with MNI Autoreg .  .  .}
	puts stdout ""
	puts stdout ". . . starting at $StartTime . . ."
	puts $logfileId ""
	puts $logfileId "Computing Talairach Transformation with MNI Autoreg .  .  ."
	puts $logfileId ""
	puts $logfileId ". . . starting at $StartTime . . ."

	set base $SUBJECTS_DIR/$subjname/mri
	set transform $base/transforms/talairach.xfm

       
	cd $SUBJECTS_DIR/$subjname/mri

	if {$VERBOSE_FLAG} {
	 puts stdout ""
	 puts stdout "Converting COR files to MINC file . . . "
	 set mri_convert_result [catch {exec $BINARIES/mri_convert orig orig/orig.mnc >@ stdout} mri_convert_msg]
         puts stdout ""
	} else {
         puts $logfileId ""
	 puts $logfileId "Converting COR files to MINC file . . . "
	 set mri_convert_result [catch {exec $BINARIES/mri_convert orig orig/orig.mnc >@ $logfileId} mri_convert_msg]
         puts stdout ""
	}
	
	# *** IMPORTANT *** must have path set to /usr/local/bin and lib on iris for this
	# perl script to find the appropriate binaries and perl libraries; work out details
	# later
	# if running on iris then perform transform on muir because
	# iris does not have the MNI software running yet
	if {$VERBOSE_FLAG} {
	    puts stdout ""
	    puts stdout "Calling mritotal . . . "
	    if { [string compare "iris.nimh.nih.gov" [exec hostname]] == 0 } {
		puts stdout "Running on iris, spawing to muir . . ."
		puts stdout "looking for stuff in $BINARIES2"
		
		set mritotal_result [catch {exec $BINARIES2/rsh muir $MNIBINARIES/mritotal -clobber -protocol icbm \
		$SUBJECTS_DIR/$subjname/mri/orig/orig.mnc $SUBJECTS_DIR/$subjname/mri/transforms/talairach.xfm >@ stdout} mritotal_msg]
	    } else {
		set mritotal_result [catch {exec $MNIBINARIES/mritotal -clobber -protocol icbm \
		orig/orig.mnc transforms/talairach.xfm >@ stdout} mritotal_msg]
	    }
	    puts stdout ""
        } else {
	    puts $logfileId ""
	    puts $logfileId "Calling mritotal . . . "
	    if { [string compare "iris.nimh.nih.gov" [exec hostname]] == 0 } {
		puts stdout "Running on iris, spawing to muir . . ."
		set mritotal_result [catch {exec $BINARIES2/rsh muir $MNIBINARIES/mritotal -clobber -protocol icbm \
		orig/orig.mnc transforms/talairach.xfm >@ $logfileId} mritotal_msg]
	    } else {	
		set mritotal_result [catch {exec $MNIBINARIES/mritotal -clobber -protocol icbm \
		$SUBJECTS_DIR/$subjname/mri/orig/orig.mnc $SUBJECTS_DIR/$subjname/mri/transforms/talairach.xfm >@ $logfileId} mritotal_msg]
	    }
	    puts $logfileId ""
	}
	
	#optionally remove the spatially normalized brain .mnc file
	#rm orig/orig.mnc

	if {$VERBOSE_FLAG} {
	 puts stdout ""
	 puts stdout "Adding Talairach transform to header . . ."
	 set mri_add_xform_to_header_result [catch {exec $BINARIES/mri_add_xform_to_header \
         $transform $base/orig $base/orig >@ stdout} mri_add_xform_to_header_msg]
	 puts stdout ""
        } else {
	 puts $logfileId ""
	 puts $logfileId "Adding Talairach transform to header . . ."
	 set mri_add_xform_to_header_result [catch {exec $BINARIES/mri_add_xform_to_header \
         $transform $base/orig $base/orig >@ $logfileId} mri_add_xform_to_header_msg]
	 puts $logfileId ""
        }

	# get the end time for processing
	set EndTime [exec date]

	puts stdout ". . . ending at $EndTime . . ."
	puts $logfileId ". . . ending at $EndTime . . ."
	puts ""

	close $logfileId

	return 0

}

#
# CorrectBiasField - 
# does full 3D intensity normalization to correct field bias
#
# this proc calls: mri_normalize and cpinfo
# 3D bias adjustment takes 15 minutes and 36 seconds on muir

proc CorrectBiasField { {BRIKNAME normalized} {HISTNAME normalizedhist.txt} } {

	global BINARIES
	global subjname
	global SUBJECTS_DIR
	global VERBOSE_FLAG
	global AFNIBIN
	global env

	set processdir [concat $SUBJECTS_DIR/$subjname]

	if [catch {open $processdir/MakeSurf.log a+ 0666} logfileId] {
	    error "CorrectBiasField: Cannot open $processdir/MakeSurf.log: $logfileId"
	}

	# CorrectBiasField needs the COR files in the orig directory
	# make check to see if they are there, if not exit
	# there should be 257 files: 256 image files plus the info file
	set origfiles [glob -nocomplain $SUBJECTS_DIR/$subjname/mri/orig/COR-*]
	if { [llength $origfiles] != 257 } {
	    error "CorrectBiasField: Insufficient number of orig COR files"
	}
	# now check the file sizes of each, each image COR  should be 65536 bytes 
	set corindex 1
	foreach file $origfiles {
	    if { $corindex > 1 } {
		set x [file size $file]
		if { $x != [expr 256*256] } {
		    error "CorrectBiasField: Improper file size for $file"
		}
	    }
	    incr corindex
	}
	
	
	# get the start time for processing
	set StartTime [exec date]

	puts stdout ""
	puts stdout {3D Intensity normalizing to correct for field bias .  .  .}
	puts stdout ""
	puts stdout ". . . starting at $StartTime . . ."
	puts $logfileId ""
	puts $logfileId "3D Intensity normalizing to correct for field bias .  .  ."
	puts $logfileId ""
	puts $logfileId ". . . starting at $StartTime . . ."


	if {$VERBOSE_FLAG} {
	 puts stdout ""
         puts stdout "Calling mri_normalize . . ."
	 set mri_normalize_result [catch {exec $BINARIES/mri_normalize $SUBJECTS_DIR/$subjname/mri/orig \
         $SUBJECTS_DIR/$subjname/mri/T1 >&@ stdout} mri_normalize_msg]
	 puts stdout ""
	} else {
	 puts $logfileId ""
         puts $logfileId "Calling mri_normalize . . . "
	 set mri_normalize_result [catch {exec $BINARIES/mri_normalize $SUBJECTS_DIR/$subjname/mri/orig \
         $SUBJECTS_DIR/$subjname/mri/T1 >&@ $logfileId} mri_normalize_msg]
	 puts $logfileId ""
	}
	
	
	# copy header information
	if {$VERBOSE_FLAG} {
	 puts stdout ""
	 puts stdout "Calling cpinfo . . ."
	 set cpinfo_result [catch {exec $BINARIES/cpinfo $subjname >&@ stdout} cpinfo_msg]
	 puts stdout ""
	} else {
	 puts $logfileId ""
	 puts $logfileId "Calling cpinfo . . ."
	 set cpinfo_result [catch {exec $BINARIES/cpinfo $subjname >&@ $logfileId} cpinfo_msg]
	 puts $logfileId ""

	}

        # create afni BRIK for intensity normalized data

	# if the BRIK exists already then remove it and create it again
	if {[file exists $SUBJECTS_DIR/$subjname/afni/normalized+orig.BRIK]} {
	     exec rm -rf $SUBJECTS_DIR/$subjname/afni/normalized+orig.BRIK
             exec rm -rf $SUBJECTS_DIR/$subjname/afni/normalized+orig.HEAD
	     exec rm -rf $SUBJECTS_DIR/$subjname/afni/normalizedhist.txt
	}
        
	if {$VERBOSE_FLAG} {
	 puts stdout ""
         puts stdout "Creating AFNI BRIK, $BRIKNAME, and histogram, $HISTNAME, for intensity normalized MR data . . ."

         set norm_result [catch {exec $AFNIBIN/to3d -spgr -prefix $BRIKNAME -session $SUBJECTS_DIR/$subjname/afni \
         -view orig -datum byte -orient LSP $SUBJECTS_DIR/$subjname/mri/T1/COR-\[0-256]\* >&@ stdout} norm_msg]

         set hist_result [catch {exec $AFNIBIN/3dhistog -nbin 255 -notit \
         $SUBJECTS_DIR/$subjname/afni/$BRIKNAME+orig.BRIK >& $SUBJECTS_DIR/$subjname/afni/$HISTNAME} hist_msg]

	 puts stdout ""
        } else {
         puts $logfileId ""
         puts $logfileId "Creating AFNI BRIK, $BRIKNAME,  and histogram, $HISTNAME, for intensity normalized MR data . . ."

         set norm_result [catch {exec $AFNIBIN/to3d -spgr -prefix $BRIKNAME -session $SUBJECTS_DIR/$subjname/afni \
         -view orig -datum byte -orient LSP $SUBJECTS_DIR/$subjname/mri/T1/COR-\[0-256]\* >&@ $logfileId} norm_msg]

         set hist_result [catch {exec $AFNIBIN/3dhistog -nbin 255 -notit \
         $SUBJECTS_DIR/$subjname/afni/$BRIKNAME+orig.BRIK >& $SUBJECTS_DIR/$subjname/afni/$HISTNAME} hist_msg] 

	 puts $logfileId ""
        }
        
	# get the end time for processing
	set EndTime [exec date]

	puts stdout ". . . ending at $EndTime . . ."
	puts $logfileId ". . . ending at $EndTime . . ."
	puts ""

	close $logfileId

        return 0
	
}

#
# StripSkull -
# performs automated skull removal
#
# this proc calls: mri_strip_skull 
# takes as arguments:
# FZERO,FMAX,DFRAC,ISTILT,mri_strip_skullmode
#
# original values that mike used for spgr on caseAH (worked well on our spgr)
# FZERO 50
# FMAX 120
# DFRAC 0.7
# ISTILT 1.0
# these are the values that Bruce recommended to be used 
# with the 2 NEX acquisitions (didn't work well!)
# FZERO 20
# FMAX 130
# DFRAC 0.65
# ISTILT 2.0
# these are the values that MGH guys are using as of 1/10/99
# FZERO 40
# FMAX 140
# DFRAC 0.7
# ISTILT 1.0


proc StripSkull { {FZERO 40} {FMAX 140} {DFRAC 0.7} {ISTILT 1.0} {mri_strip_skullmode 1} } {

	global BINARIES
	global subjname
	global SUBJECTS_DIR
	global MRI_DIR
	global VERBOSE_FLAG
	global AFNIBIN
	global env
	
	set processdir [concat $SUBJECTS_DIR/$subjname]

	if [catch {open $processdir/MakeSurf.log a+ 0666} logfileId] {
	    error "StripSkull: Cannot open $processdir/MakeSurf.log: $logfileId"
	}

	# StripSkull needs the COR files in the T1 directory
	# make check to see if they are there, if not exit
	# there should be 257 files: 256 image files plus the info file
	set T1files [glob -nocomplain $SUBJECTS_DIR/$subjname/mri/T1/COR-*]
	if { [llength $T1files] != 257 } {
	    error "StripSkull: Insufficient number of T1 COR files"
	}
	# now check the file sizes of each, each image COR should be 65536 bytes 
	set corindex 1
	foreach file $T1files {
	    if { $corindex > 1 } {
		set x [file size $file]
		if { $x != [expr 256*256] } {
		    error "StripSkull: Improper file size for $file"
		}
	    }
	    incr corindex
	}
	
	# get the start time for processing
	set StartTime [exec date]

	# go to the directory where brain.dat will
	# reside. This is very important since
	# mri_strip_skull looks in the directory from
	# which it is executed for the brain.dat
	# file

	cd $SUBJECTS_DIR/$subjname/mri

	puts stdout ""
	puts stdout {Performing automated skull removal .  .  .}
	puts stdout ""
	puts stdout ". . . starting at $StartTime . . ."
	puts $logfileId ""
	puts $logfileId "Performing automated skull removal .  .  ."
	puts $logfileId ""
	puts $logfileId ". . . starting at $StartTime . . ."


	# *** IMPORTANT *** set the environment variable MRI_DIR
	# so that the contents of the bem directory are available
	# in particular the triangle file 

	set env(MRI_DIR) $MRI_DIR

	# *** IMPORTANT *** mri_strip_skull also needs the SUBJECTS_DIR 
	# environment variable set

	set env(SUBJECTS_DIR) $SUBJECTS_DIR

	# mri_strip_skull needs the brain.dat file to be in the current working directory
	# create the brain.dat file; open for writing

	set brain_dat_fid [open $SUBJECTS_DIR/$subjname/mri/brain.dat w+ 0666]

        puts $brain_dat_fid "fzero $FZERO"
	puts $brain_dat_fid "fmax $FMAX"
	puts $brain_dat_fid "dfrac $DFRAC"
	puts $brain_dat_fid "istilt $ISTILT"

        #puts $brain_dat_fid "fzero 25"
	#puts $brain_dat_fid "fmax 140"
	#puts $brain_dat_fid "dfrac 0.65"
	#puts $brain_dat_fid "istilt 1.0"

	close $brain_dat_fid

	# do the actual stripping with mri_strip_skull

	if {$VERBOSE_FLAG} {
	 puts stdout ""
	 puts stdout "Calling mri_strip_skull . . ."
	 set mri_strip_skull_result [catch {exec $BINARIES/mri_strip_skull $subjname $mri_strip_skullmode >&@ stdout} mri_strip_skull_msg]
	 puts stdout ""
        } else {
	 puts $logfileId ""
	 puts $logfileId "Calling mri_strip_skull . . ."
         set mri_strip_skull_result [catch {exec $BINARIES/mri_strip_skull $subjname $mri_strip_skullmode >&@ $logfileId} mri_strip_skull_msg]
	 puts $logfileId ""
	}

        # create afni BRIK for skull stripped data

	# if the BRIK exists already then remove it and create it again
	if {[file exists $SUBJECTS_DIR/$subjname/afni/brain+orig.BRIK]} {
	     exec rm -rf $SUBJECTS_DIR/$subjname/afni/brain+orig.BRIK
             exec rm -rf $SUBJECTS_DIR/$subjname/afni/brain+orig.HEAD
	     exec rm -rf $SUBJECTS_DIR/$subjname/afni/brainhist.txt
	}

        if {$VERBOSE_FLAG} {
	 puts stdout ""
         puts stdout "Creating AFNI BRIK and histogram for mri_strip_skull output . . ."

         set strip_result [catch {exec $AFNIBIN/to3d -spgr -prefix brain -session $SUBJECTS_DIR/$subjname/afni -view orig \
         -datum byte -orient LSP $SUBJECTS_DIR/$subjname/mri/brain/COR-\[0-256]\* >&@ stdout} strip_msg]

	 set hist_result [catch {exec $AFNIBIN/3dhistog -nbin 255 -notit \
         $SUBJECTS_DIR/$subjname/afni/brain+orig.BRIK >& $SUBJECTS_DIR/$subjname/afni/brainhist.txt} hist_msg]

         puts stdout ""
        } else {
	 puts $logfileId ""
         puts $logfileId "Creating AFNI BRIK and histogram for mri_strip_skull output . . ."

         set strip_result [catch {exec $AFNIBIN/to3d -spgr -prefix brain -session $SUBJECTS_DIR/$subjname/afni -view orig \
         -datum byte -orient LSP $SUBJECTS_DIR/$subjname/mri/brain/COR-\[0-256]\* >&@ $logfileId} strip_msg]

         set hist_result [catch {exec $AFNIBIN/3dhistog -nbin 255 -notit \
         $SUBJECTS_DIR/$subjname/afni/brain+orig.BRIK >& $SUBJECTS_DIR/$subjname/afni/brainhist.txt} hist_msg]

         puts $logfileId ""
        }

	# get the end time for processing
	set EndTime [exec date]

	puts stdout ". . . ending at $EndTime . . ."
	puts $logfileId ". . . ending at $EndTime . . ."
	puts ""
	
	close $logfileId

        return 0

}

#
# ClassifyWM - 
# classifies white matter
#
# this proc calls: wmfilter
# takes as arguments:
#
# WHITE_HILIM
# WHITE_LOLIM
# GRAY_HILIM
# 


proc ClassifyWM { {WHITE_HILIM 130} {WHITE_LOLIM 90} {GRAY_HILIM 100} } {

	global BINARIES
	global subjname
	global SUBJECTS_DIR
	global VERBOSE_FLAG
	global AFNIBIN
	global env
	
	set processdir [concat $SUBJECTS_DIR/$subjname]

	if [catch {open $processdir/MakeSurf.log a+ 0666} logfileId] {
	    error "ClassifyWM: Cannot open $processdir/MakeSurf.log: $logfileId"
	}

	# StripSkull needs the COR files in the brain directory
	# make check to see if they are there, if not exit
	# there should be 257 files: 256 image files plus the info file
	#set brainfiles [glob -nocomplain $SUBJECTS_DIR/$subjname/mri/brain/COR-*]
	#if { [llength $brainfiles] != 257 } {
	#    error "ClassifyWM: Insufficient number of brain COR files"
	#}
	# now check the file sizes of each, each image COR should be 65536 bytes 
	#set corindex 1
	#foreach file $brainfiles {
	#    if { $corindex > 1 } {
	#	set x [file size $file]
	#	if { $x != [expr 256*256] } {
	#	    error "ClassifyWM: Improper file size for $file"
	#	}
	#    }
	#    incr corindex
	#}
	
	# get the start time for processing
	set StartTime [exec date]

	# go to the directory where wmfilter.dat will
	# reside. This is very important since
	# wmfilter looks in the directory from
	# which it is executed for the wmfilter.dat
	# file

	cd $SUBJECTS_DIR/$subjname/mri

	puts stdout ""
	puts stdout {Classifying tissue types .  .  .}
	puts stdout ""
	puts stdout ". . . starting at $StartTime . . ."
	puts $logfileId ""
	puts $logfileId "Classifying tissue types .  .  ."
	puts $logfileId ""
	puts $logfileId ". . . starting at $StartTime . . ."

	# wmfilter needs wmfilter.dat in the current working directory
	# we create it here on-the-fly (as with brain.dat) 

	set wmfilter_dat_fid [open $SUBJECTS_DIR/$subjname/mri/wmfilter.dat w+ 0666]

	puts $wmfilter_dat_fid "white_hilim $WHITE_HILIM"
	puts $wmfilter_dat_fid "white_lolim $WHITE_LOLIM"
	puts $wmfilter_dat_fid "gray_hilim $GRAY_HILIM"
 
        # testing surface generation on a 2NEX volume
        #puts $wmfilter_dat_fid "white_hilim 130"
	#puts $wmfilter_dat_fid "white_lolim 75"
	#puts $wmfilter_dat_fid "gray_hilim 85"

	close $wmfilter_dat_fid

	# perform white matter classification with wmfilter

	if {$VERBOSE_FLAG} {
	 puts stdout ""
	 puts stdout "Calling wmfilter . . ."
	 set wmfilter_result [catch {exec $BINARIES/mri_wmfilter $subjname >&@ stdout} wmfilter_msg]
	 puts stdout ""
	 puts stdout "$wmfilter_msg and $wmfilter_result"
        } else {
	 puts $logfileId ""
	 puts $logfileId "Calling wmfilter . . ."
         set wmfilter_result [catch {exec $BINARIES/mri_wmfilter $subjname >&@ $logfileId} wmfilter_msg]
	 puts $logfileId ""
	}

	
	# create afni BRIK for white matter classification output

	# if the BRIK exists already then remove it and create it again
	if {[file exists $SUBJECTS_DIR/$subjname/afni/wmfilter+orig.BRIK]} {
	     exec rm -rf $SUBJECTS_DIR/$subjname/afni/wmfilter+orig.BRIK
             exec rm -rf $SUBJECTS_DIR/$subjname/afni/wmfilter+orig.HEAD
	     exec rm -rf $SUBJECTS_DIR/$subjname/afni/wmfilterhist.txt
	}

	# create afni BRIK for skull stripped data
        if {$VERBOSE_FLAG} {
	 puts stdout ""
         puts stdout "Creating AFNI BRIK and histogram for wmfilter output . . ."

         set wm_result [catch {exec $AFNIBIN/to3d -spgr -prefix wmfilter -session $SUBJECTS_DIR/$subjname/afni \
         -view orig -datum byte -orient LSP $SUBJECTS_DIR/$subjname/mri/wm/COR-\[0-256]\* >&@ stdout} wm_msg]

	 set hist_result [catch {exec $AFNIBIN/3dhistog -nbin 255 -notit \
         $SUBJECTS_DIR/$subjname/afni/wmfilter+orig.BRIK >& $SUBJECTS_DIR/$subjname/afni/wmfilterhist.txt} hist_msg]

         puts stdout ""
        } else {
	 puts $logfileId ""
         puts $logfileId "Creating AFNI BRIK and histogram for wmfilter output . . ."

         set wm_result [catch {exec $AFNIBIN/to3d -spgr -prefix wmfilter -session $SUBJECTS_DIR/$subjname/afni \
         -view orig datum byte -orient LSP $SUBJECTS_DIR/$subjname/mri/wm/COR-\[0-256]\* >&@ $logfileId} wm_msg]

         set hist_result [catch {exec $AFNIBIN/3dhistog -nbin 255 -notit \
         $SUBJECTS_DIR/$subjname/afni/wmfilter+orig.BRIK >& $SUBJECTS_DIR/$subjname/afni/wmfilterhist.txt} hist_msg]

         puts $logfileId ""
        }


	# get the end time for processing
	set EndTime [exec date]

	puts stdout ". . . ending at $EndTime . . ."
	puts $logfileId ". . . ending at $EndTime . . ."
	puts ""

	close $logfileId

        return 0

}

#
# FillBrain -
# establishes cutting planes and fills brain volume
#
# this proc calls: mri_fill
#
#

proc FillBrain { } {

	global BINARIES
	global subjname
	global SUBJECTS_DIR
	global VERBOSE_FLAG
	global AFNIBIN
	global env
	
	set processdir [concat $SUBJECTS_DIR/$subjname]

	if [catch {open $processdir/MakeSurf.log a+ 0666} logfileId] {
	    error "FillBrain: Cannot open $processdir/MakeSurf.log: $logfileId"
	}

	# FillBrain needs the COR files in the wm directory
	# make check to see if they are there, if not exit
	# there should be 257 files: 256 image files plus the info file
	#set wmfiles [glob -nocomplain $SUBJECTS_DIR/$subjname/mri/wm/COR-*]
	#if { [llength $wmfiles] != 257 } {
	#    error "FillBrain: Insufficient number of wm COR files"
	#}
	# now check the file sizes of each, each image COR should be 65536 bytes 
	#set corindex 1
	#foreach file $wmfiles {
	#    if { $corindex > 1 } {
	#	set x [file size $file]
	#	if { $x != [expr 256*256] } {
	#	    error "FillBrain: Improper file size for $file"
	#	}
	#    }
	#    incr corindex
	#}
	
	# get the start time for processing
	set StartTime [exec date]

	puts stdout ""
	puts stdout {Establishing cutting planes and filling the brain volume .  .  .}
	puts stdout ""
	puts stdout ". . . starting at $StartTime . . ."
	puts $logfileId ""
	puts $logfileId "Establishing cutting planes and filling the brain volume .  .  ."
	puts $logfileId ""
	puts $logfileId ". . . starting at $StartTime . . ."

        if {([file exists $SUBJECTS_DIR/$subjname/mri/transforms/talairach.xfm] == 0)} {
	    puts stderr "FillBrain - cannot find talairach transform file"
            exit
	}
	
	# when operating in verbose diagnostic mode *.mnc files are output
	# they take up space and currently have no use. Check for them and 
	# remove if necessary
	set mncjunk [glob -nocomplain *.mnc]
	if [llength $mncjunk] {
	    puts stdout ""
	    puts $logfileId ""
	    puts stdout "Removing mnc diagnostic output"
	    puts $logfileId "Removing mnc diagnostic output"
	    exec rm -rf *.mnc
	}

	

	# operate in the "wm" directory, put 
	# results in the "filled" directory

	if {$VERBOSE_FLAG} {
	 puts stdout ""
	 puts stdout "Calling mri_fill . . ."
	 set mri_fill_result [catch {exec $BINARIES/mri_fill $SUBJECTS_DIR/$subjname/mri/wm \
	 $SUBJECTS_DIR/$subjname/mri/filled >&@ stdout} mri_fill_msg] 
	 puts stdout ""
        } else {
	 puts $logfileId ""
	 puts $logfileId "Calling mri_fill . . ."
         set mri_fill_result [catch {exec $BINARIES/mri_fill $SUBJECTS_DIR/$subjname/mri/wm \
	 $SUBJECTS_DIR/$subjname/mri/filled >&@ $logfileId} mri_fill_msg] 
	 puts $logfileId ""
	}

	# create afni BRIK for filled hemispheres
        # this hangs if user does not have permission to remove!!!!
        # make sure this is fixed at a later date
	# if the BRIK exists already then remove it and create it again
	if {[file exists $SUBJECTS_DIR/$subjname/afni/filled+orig.BRIK]} {
             puts stdout "Removing old BRIKS . . ." 
	     exec rm -rf $SUBJECTS_DIR/$subjname/afni/filled+orig.BRIK
             exec rm -rf $SUBJECTS_DIR/$subjname/afni/filled+orig.HEAD
	     puts stdout "Done Removing old BRIKS." 
	}

        if {$VERBOSE_FLAG} {
	 puts stdout ""
         puts stdout "Creating AFNI BRIK for filled hemispheres . . ."
         set filled_result [catch {exec $AFNIBIN/to3d -spgr -prefix filled -session $SUBJECTS_DIR/$subjname/afni \
         -view orig -datum byte -orient LSP $SUBJECTS_DIR/$subjname/mri/filled/COR-\[0-256]\* >&@ stdout} filled_msg]
         puts stdout ""
        } else {
	 puts $logfileId ""
         puts $logfileId "Creating AFNI BRIK for filled hemispheres . . ."
         set filled_result [catch {exec $AFNIBIN/to3d -spgr -prefix filled -session $SUBJECTS_DIR/$subjname/afni \
         -view orig -datum byte -orient LSP $SUBJECTS_DIR/$subjname/mri/filled/COR-\[0-256]\* >&@ $logfileId} filled_msg]
         puts $logfileId ""
        }

        # when operating in verbose diagnostic mode *.mnc files are output
	# they take up space and currently have no use. Check for them and 
	# remove if necessary
	set mncjunk [glob -nocomplain *.mnc]
	if [llength $mncjunk] {
	    puts stdout ""
	    puts $logfileId ""
	    puts stdout "Removing mnc diagnostic output"
	    puts $logfileId "Removing mnc diagnostic output"
	    exec rm -rf *.mnc
	}

	# get the end time for processing
	set EndTime [exec date]

	puts stdout ". . . ending at $EndTime . . ."
	puts $logfileId ". . . ending at $EndTime . . ."
	puts ""

	close $logfileId
	
}

#
# ReconstructSurface -
# reconstructs the surface separately for left and right hemispheres
#
# this section of code calls: mri_tessellate, mris_smooth
# 1 - right hemisphere reconstruction
# 2 - left hemipshere reconstruction

proc ReconstructSurface { {WhichHemisphere 1} } {

	global BINARIES
	global subjname
	global SUBJECTS_DIR
	global VERBOSE_FLAG
	global AFNIBIN
	global env
	
	set processdir [concat $SUBJECTS_DIR/$subjname]
	
	if [catch {open $processdir/MakeSurf.log a+ 0666} logfileId] {
	    error "ReconstructSurface - Cannot open $processdir/MakeSurf.log: $logfileId"
	}
	
	# FillBrain needs the COR files in the wm directory
	# make check to see if they are there, if not exit
	# there should be 257 files: 256 image files plus the info file
	set filledfiles [glob -nocomplain $SUBJECTS_DIR/$subjname/mri/filled/COR-*]
	if { [llength $filledfiles] != 257 } {
	    error "ReconstructSurface: Insufficient number of filled COR files"
	}
	# now check the file sizes of each, each image COR should be 65536 bytes 
	set corindex 1
	foreach file $filledfiles {
	    if { $corindex > 1 } {
		set x [file size $file]
		if { $x != [expr 256*256] } {
		    error "ReconstructSurface: Improper file size for $file"
		}
	    }
	    incr corindex
	}
	
	
	# get the start time for processing
	set StartTime [exec date]

	puts stdout ""
	puts stdout {Reconstructing the surface .  .  .}
	puts stdout ""
	puts stdout ". . . starting at $StartTime . . ."
	puts $logfileId ""
	puts $logfileId "Reconstructing the surface .  .  ."
	puts $logfileId ""
	puts $logfileId ". . . starting at $StartTime . . ."

	# go to the surf dir
	cd $SUBJECTS_DIR/$subjname/surf

	switch -exact -- $WhichHemisphere {
	
	    1 {

		if {$VERBOSE_FLAG} {
		    puts stdout ""
		    puts stdout "Reconstructing right hemisphere surface . . ."
		    set tess_rh_result [catch {exec $BINARIES/mri_tessellate ../mri/filled \
                       127 rh.orig >&@ stdout} tess_rh_msg]
                    set smoothwm_rh_result [catch {exec $BINARIES/mris_smooth rh.orig \
                       rh.smoothwm >&@ stdout} smoothwm_rh_msg]
		    puts stdout ""
		} else {
		    puts $logfileId ""
		    puts $logfileId "Reconstructing right hemisphere surface . . ."
		    set tess_rh_result [catch {exec $BINARIES/mri_tessellate ../mri/filled \
                       127 rh.orig >&@ $logfileId} tess_rh_msg]
                    set smoothwm_rh_result [catch {exec $BINARIES/mris_smooth rh.orig \
                       rh.smoothwm >&@ $logfileId} smoothwm_rh_msg]
		    puts $logfileId ""
		}

	    }

	    2 {

		if {$VERBOSE_FLAG} {
		    puts stdout ""
		    puts stdout "Reconstructing left hemisphere surface . . ."
		    set tess_lh_result [catch {exec $BINARIES/mri_tessellate ../mri/filled \
                      255 lh.orig >&@ stdout} tess_lh_msg]
                    set smoothwm_lh_result [catch {exec $BINARIES/mris_smooth lh.orig \
                       lh.smoothwm >&@ stdout} smoothwm_lh_msg]
		    puts stdout ""
		} else {
		    puts $logfileId ""
		    puts $logfileId "Reconstructing left hemisphere surface . . ."
		    set tess_lh_result [catch {exec $BINARIES/mri_tessellate ../mri/filled \
                      255 lh.orig >&@ $logfileId} tess_lh_msg]
                    set smoothwm_lh_result [catch {exec $BINARIES/mris_smooth lh.orig \
                       lh.smoothwm >&@ $logfileId} smoothwm_lh_msg]
		    puts $logfileId ""
		}

	    }

	    default {

		if {$VERBOSE_FLAG} {
		    puts stdout ""
		    puts stdout "Reconstructing right hemisphere surface . . ."
		    set tess_rh_result [catch {exec $BINARIES/mri_tessellate ../mri/filled \
                       127 rh.orig >&@ stdout} tess_rh_msg]
                    set smoothwm_rh_result [catch {exec $BINARIES/mris_smooth rh.orig \
                       rh.smoothwm >&@ stdout} smoothwm_rh_msg]
		    puts stdout ""
		} else {
		    puts $logfileId ""
		    puts $logfileId "Reconstructing right hemisphere surface . . ."
		    set tess_rh_result [catch {exec $BINARIES/mri_tessellate ../mri/filled \
                       127 rh.orig >&@ $logfileId} tess_rh_msg]
                    set smoothwm_rh_result [catch {exec $BINARIES/mris_smooth rh.orig \
                       rh.smoothwm >&@ $logfileId} smoothwm_rh_msg]
		    puts $logfileId ""
		}

		
	    }

	}

	# get the end time for processing
	set EndTime [exec date]

	puts stdout ". . . ending at $EndTime . . ."
	puts $logfileId ". . . ending at $EndTime . . ."
	puts ""

	# don't forget to close the logfile!	
	close $logfileId

	return 0

}

#
# does preliminary surface inflation for use in the editing loop
# use MakeFinalSurfaces to create the final inflation after successful and complete editing
#

proc InflateSurface { {WhichHemisphere 1} } {

    global BINARIES
    global subjname
    global SUBJECTS_DIR
    global VERBOSE_FLAG
    global AFNIBIN
    global env
	
    set processdir [concat $SUBJECTS_DIR/$subjname]

    if [catch {open $processdir/MakeSurf.log a+ 0666} logfileId] {
	error "InflateSurface: Cannot open $processdir/MakeSurf.log: $logfileId"
    }

    	
    # get the start time for processing
    set StartTime [exec date]

    puts stdout ""
    puts stdout {Inflating the surface reconstruction.  .  .}
    puts stdout ""
    puts stdout ". . . starting at $StartTime . . ."
    puts $logfileId ""
    puts $logfileId "Inflating the surface reconstruction .  .  ."
    puts $logfileId ""
    puts $logfileId ". . . starting at $StartTime . . ."

    # go to the surf dir
    cd $SUBJECTS_DIR/$subjname/surf

    switch -exact -- $WhichHemisphere {
	
	    1 {

		# InflateSurface needs the .smoothwm file to operate properly
		# check to see if it exists in the surface directory, if not exit
		if { ![file exists $SUBJECTS_DIR/$subjname/surf/rh.smoothwm] } {
		    error "InflateSurface: cannot find rh.smoothwm surface file"
		}

		if {$VERBOSE_FLAG} {
		    puts stdout ""
		    puts stdout "Inflating the right hemisphere surface . . ."
		    set inflate_rh_result [catch {exec $BINARIES/mris_inflate rh.smoothwm \
                    rh.inflated >&@ stdout} inflate_rh_msg]
		    puts stdout ""
		} else {
		    puts $logfileId ""
		    puts $logfileId "Inflating the right hemisphere surface . . ."
		    set inflate_rh_result [catch {exec $BINARIES/mris_inflate rh.smoothwm \
                    rh.inflated >&@ $logfileId} inflate_rh_msg]
		    puts $logfileId ""
		}

		# check to see if inflated output has been written to disk
		if { ![file exists $SUBJECTS_DIR/$subjname/surf/rh.inflated] } {
		    error "InflateSurface: failed to write rh.inflated surface file"
		}

	    }

	    2 {

		# InflateSurface needs the .smoothwm file to operate properly
		# check to see if it exists in the surface directory, if not exit
		if { ![file exists $SUBJECTS_DIR/$subjname/surf/lh.smoothwm] } {
		    error "InflateSurface: cannot find lh.smoothwm surface file"
		}

		if {$VERBOSE_FLAG} {
		    puts stdout ""
		    puts stdout "Inflating the left hemisphere surface . . ."
		    set inflate_lh_result [catch {exec $BINARIES/mris_inflate lh.smoothwm \
                    lh.inflated >&@ stdout} inflate_lh_msg]
		    puts stdout ""
		} else {
		    puts $logfileId ""
		    puts $logfileId "Inflating the left hemisphere surface . . ."
		    set inflate_lh_result [catch {exec $BINARIES/mris_inflate lh.smoothwm \
                    lh.inflated >&@ $logfileId} inflate_lh_msg]
		    puts $logfileId ""
		}

		# check to see if inflated output has been written to disk
		if { ![file exists $SUBJECTS_DIR/$subjname/surf/lh.inflated] } {
		    error "InflateSurface: failed to write lh.inflated surface file"
		}

	    }

	    default {

		if { ![file exists $SUBJECTS_DIR/$subjname/surf/rh.smoothwm] } {
		    error "InflateSurface: cannot find rh.smoothwm surface file"
		}

		if {$VERBOSE_FLAG} {
		    puts stdout ""
		    puts stdout "Inflating the right hemisphere surface . . ."
		    set inflate_rh_result [catch {exec $BINARIES/mris_inflate rh.smoothwm \
                    rh.inflated >&@ stdout} inflate_rh_msg]
		    puts stdout ""
		} else {
		    puts $logfileId ""
		    puts $logfileId "Inflating the right hemisphere surface . . ."
		    set inflate_rh_result [catch {exec $BINARIES/mris_inflate rh.smoothwm \
                    rh.inflated >&@ $logfileId} inflate_rh_msg]
		    puts $logfileId ""
		}

		# check to see if inflated output has been written to disk
		if { ![file exists $SUBJECTS_DIR/$subjname/surf/rh.inflated] } {
		    error "InflateSurface: failed to write rh.inflated surface file"
		}

	    }

	}

    # get the end time for processing
    set EndTime [exec date]

    puts stdout ". . . ending at $EndTime . . ."
    puts $logfileId ". . . ending at $EndTime . . ."
    puts ""

    # don't forget to close the logfile!
    close $logfileId

    return 0

}


#
# launches tksurfer and tkmedit for surface editing
#
#

proc EditSurface { {WhichHemisphere 1} } {

    global BINARIES
    global BINARIES2
    global subjname
    global SUBJECTS_DIR
    global VERBOSE_FLAG
    global AFNIBIN
    global env
	
    set processdir [concat $SUBJECTS_DIR/$subjname]

    if [catch {open $processdir/MakeSurf.log a+ 0666} logfileId] {
	error "EditSurface: Cannot open $processdir/MakeSurf.log: $logfileId"
    }

    
    # EditSurface needs the COR files in the wm directory
    # make check to see if they are there, if not exit
    # there should be 257 files: 256 image files plus the info file
    set wmfiles [glob -nocomplain $SUBJECTS_DIR/$subjname/mri/wm/COR-*]
    if { [llength $wmfiles] != 257 } {
	error "EditSurface: Insufficient number of wm COR files"
    }
    # now check the file sizes of each, each image COR should be 65536 bytes 
    set corindex 1
    foreach file $wmfiles {
	if { $corindex > 1 } {
	    set x [file size $file]
	    if { $x != [expr 256*256] } {
		error "EditSurface: Improper file size for $file"
	    }
	}
	incr corindex
    }
	
    
    # get the start time for processing
    set StartTime [exec date]

    puts stdout ""
    puts stdout {Editing the surface reconstruction.  .  .}
    puts stdout ""
    puts stdout ". . . starting at $StartTime . . ."
    puts $logfileId ""
    puts $logfileId "Editing the surface reconstruction .  .  ."
    puts $logfileId ""
    puts $logfileId ". . . starting at $StartTime . . ."

    # go to the surf dir
    cd $SUBJECTS_DIR/$subjname/scripts

    # use switch to choose which hemisphere to edit

    switch -exact -- $WhichHemisphere {
	
	1 {

	    # right hemisphere

	    # check for existence of necessary input surface files 
	    if { ![file exists $SUBJECTS_DIR/$subjname/surf/rh.orig] } {
		    error "EditSurface: cannot find rh.orig surface file"
	    }
	    if { ![file exists $SUBJECTS_DIR/$subjname/surf/rh.inflated] } {
		    error "EditSurface: cannot find rh.inflated surface file"
	    }

	    puts stdout ""
	    puts stdout "Editing right hemisphere . . . "
	    puts stdout ""

	    if {$VERBOSE_FLAG} {
		puts stdout ""
		puts stdout "Editing the right hemisphere surface . . ."
		
		set tksurfer_rh_result [catch {exec $BINARIES/tksurfer -$subjname rh inflated & } tksurfer_rh_msg]

		# if running on iris then run tkmedit on muir because there is 
		# unexplained problem writing edited COR files in tkmedit from iris 
		if { [string compare "iris.nimh.nih.gov" [exec hostname]] == 0 } {
		    set cmd " setenv DISPLAY iris.nimh.nih.gov:0.0; setenv SUBJECTS_DIR $SUBJECTS_DIR; cd $SUBJECTS_DIR/$subjname/scripts; $BINARIES/tkmedit $subjname wm 2 rh.orig "
		    set tkmedit_rh_result [catch {exec $BINARIES2/remotetkmedit muir [format "%s" $cmd]   >@ stdout } tkmedit_rh_msg]
		} else {
		    set tkmedit_rh_result [catch {exec $BINARIES/tkmedit $subjname wm 2 rh.orig & } tkmedit_rh_msg]
		    puts stdout ""
		}
	    } else {
		puts $logfileId ""
		puts $logfileId "Editing the right hemisphere surface . . ."

		set tksurfer_rh_result [catch {exec $BINARIES/tksurfer -$subjname rh inflated & } tksurfer_rh_msg]

		# if running on iris then run tkmedit on muir because there is 
		# unexplained problem writing edited COR files in tkmedit from iris 
		if { [string compare "iris.nimh.nih.gov" [exec hostname]] == 0 } {
		    puts stdout "Running on iris, spawing to muir . . ."
		    set cmd " setenv DISPLAY iris.nimh.nih.gov:0.0; setenv SUBJECTS_DIR $SUBJECTS_DIR; cd $SUBJECTS_DIR/$subjname/scripts; $BINARIES/tkmedit $subjname wm 2 rh.orig "
		    set tkmedit_rh_result [catch {exec $BINARIES2/remotetkmedit muir [format "%s" $cmd]  >@ stdout } tkmedit_rh_msg]
		    
		} else {
		    set tkmedit_rh_result [catch {exec $BINARIES/tkmedit $subjname wm 2 rh.orig & } tkmedit_rh_msg]
		    puts stdout ""
		}
		
		puts $logfileId ""
	    }	
		  	
	}

	2 {

	    # left hemisphere

	    # check for existence of necessary input surface files 
	    if { ![file exists $SUBJECTS_DIR/$subjname/surf/lh.orig] } {
		    error "EditSurface: cannot find lh.orig surface file"
	    }
	    if { ![file exists $SUBJECTS_DIR/$subjname/surf/lh.inflated] } {
		    error "EditSurface: cannot find lh.inflated surface file"
	    }

	    puts stdout ""
	    puts stdout "Editing left hemisphere . . . "
	    puts stdout ""

	    if {$VERBOSE_FLAG} {
		puts stdout ""
		puts stdout "Editing the left hemisphere surface . . ."

		set tksurfer_lh_result [catch {exec $BINARIES/tksurfer -$subjname lh inflated & } tksurfer_lh_msg]

		# if running on iris then run tkmedit on muir because there is 
		# unexplained problem writing edited COR files in tkmedit from iris 
		if { [string compare "iris.nimh.nih.gov" [exec hostname]] == 0 } {
		    set cmd " setenv DISPLAY iris.nimh.nih.gov:0.0; setenv SUBJECTS_DIR $SUBJECTS_DIR; cd $SUBJECTS_DIR/$subjname/scripts; $BINARIES/tkmedit $subjname wm 2 lh.orig "
		    set tkmedit_lh_result [catch {exec $BINARIES2/remotetkmedit muir [format "%s" $cmd]   >@ stdout } tkmedit_lh_msg]
		    
		} else {
		    set tkmedit_lh_result [catch {exec $BINARIES/tkmedit $subjname wm 2 lh.orig  } tkmedit_lh_msg]
		    puts stdout ""
		}
		
	    } else {
		puts $logfileId ""
		puts $logfileId "Editing the left hemisphere surface . . ."

		set tksurfer_lh_result [catch {exec $BINARIES/tksurfer -$subjname lh inflated & } tksurfer_lh_msg]

		# if running on iris then run tkmedit on muir because there is 
		# unexplained problem writing edited COR files in tkmedit from iris 
		if { [string compare "iris.nimh.nih.gov" [exec hostname]] == 0 } {
		    set cmd " setenv DISPLAY iris.nimh.nih.gov:0.0; setenv SUBJECTS_DIR $SUBJECTS_DIR; cd $SUBJECTS_DIR/$subjname/scripts; $BINARIES/tkmedit $subjname wm 2 lh.orig "
		    set tkmedit_lh_result [catch {exec $BINARIES2/remotetkmedit muir [format "%s" $cmd]   >@ stdout } tkmedit_lh_msg]
		} else {
		    set tkmedit_lh_result [catch {exec $BINARIES/tkmedit $subjname wm 2 lh.orig & } tkmedit_lh_msg]
		    puts $logfileId ""
		}

		
	    }


	}

	default {

	    # right hemisphere

	    # check for existence of necessary input surface files 
	    if { ![file exists $SUBJECTS_DIR/$subjname/surf/rh.orig] } {
		    error "EditSurface: cannot find rh.orig surface file"
	    }
	    if { ![file exists $SUBJECTS_DIR/$subjname/surf/rh.inflated] } {
		    error "EditSurface: cannot find rh.inflated surface file"
	    }

	    puts stdout ""
	    puts stdout "Editing right hemisphere . . . "
	    puts stdout ""

	    if {$VERBOSE_FLAG} {
		puts stdout ""
		puts stdout "Editing the right hemisphere surface . . ."
		
		set tksurfer_rh_result [catch {exec $BINARIES/tksurfer -$subjname rh inflated & } tksurfer_rh_msg]

		# if running on iris then run tkmedit on muir because there is 
		# unexplained problem writing edited COR files in tkmedit from iris 
		if { [string compare "iris.nimh.nih.gov" [exec hostname]] == 0 } {
		    set cmd " setenv DISPLAY iris.nimh.nih.gov:0.0; setenv SUBJECTS_DIR $SUBJECTS_DIR; cd $SUBJECTS_DIR/$subjname/scripts; $BINARIES/tkmedit $subjname wm 2 rh.orig "
		    set tkmedit_rh_result [catch {exec $BINARIES2/remotetkmedit muir [format "%s" $cmd] >@ stdout } tkmedit_rh_msg]
		} else {
		    set tkmedit_rh_result [catch {exec $BINARIES/tkmedit $subjname wm 2 rh.orig & } tkmedit_rh_msg]
		    puts stdout ""
		}
	    } else {
		puts $logfileId ""
		puts $logfileId "Editing the right hemisphere surface . . ."

		set tksurfer_rh_result [catch {exec $BINARIES/tksurfer -$subjname rh inflated & } tksurfer_rh_msg]

		# if running on iris then run tkmedit on muir because there is 
		# unexplained problem writing edited COR files in tkmedit from iris 
		if { [string compare "iris.nimh.nih.gov" [exec hostname]] == 0 } {
		    set cmd " setenv DISPLAY iris.nimh.nih.gov:0.0; setenv SUBJECTS_DIR $SUBJECTS_DIR; cd $SUBJECTS_DIR/$subjname/scripts; $BINARIES/tkmedit $subjname wm 2 rh.orig "
		    set tkmedit_rh_result [catch {exec $BINARIES2/remotetkmedit muir [format "%s" $cmd] >@ stdout } tkmedit_rh_msg]
		} else {
		    set tkmedit_rh_result [catch {exec $BINARIES/tkmedit $subjname wm 2 rh.orig & } tkmedit_rh_msg]
		    puts stdout ""
		}
		
		puts $logfileId ""
	    }
			
	}

    }


    # get the end time for processing
    set EndTime [exec date]

    puts stdout ". . . ending at $EndTime . . ."
    puts $logfileId ". . . ending at $EndTime . . ."
    puts ""

    # don't forget to close the logfile!
    close $logfileId

    

    return 0

}

proc MakeFinalSurfaces { {WhichHemisphere 1} } {


    global BINARIES
    global subjname
    global SUBJECTS_DIR
    global VERBOSE_FLAG
    global AFNIBIN
    global env
	
    set processdir [concat $SUBJECTS_DIR/$subjname]

    if [catch {open $processdir/MakeSurf.log a+ 0666} logfileId] {
	error "MakeFinalSurfaces: Cannot open $processdir/MakeSurf.log: $logfileId"
    }
	
    	
    # get the start time for processing
    set StartTime [exec date]

    puts stdout ""
    puts stdout {Reconstructing the final surfaces  .  .  .}
    puts stdout ""
    puts stdout ". . . starting at $StartTime . . ."
    puts $logfileId ""
    puts $logfileId "Reconstructing the final surfaces  .  .  ."
    puts $logfileId ""
    puts $logfileId ". . . starting at $StartTime . . ."

    # go to the surf dir
    cd $SUBJECTS_DIR/$subjname/surf

    switch -exact -- $WhichHemisphere {
	
	    1 {

		# check for existence of inflated surface
		if { ![file exists $SUBJECTS_DIR/$subjname/surf/rh.inflated] } {
		    error "MakeFinalSurfaces: can't find rh.inflated surface file"
		}

		if {$VERBOSE_FLAG} {
		    puts stdout ""
		    puts stdout "Reconstructing the final right hemisphere surfaces . . ."
		    set rh_result [catch {exec $BINARIES/mris_make_surfaces -q $subjname rh  >&@ stdout} rh_msg]
		    puts stdout ""
		} else {
		    puts $logfileId ""
		    puts $logfileId "Reconstructing the final right hemisphere surfaces . . ."
		    set rh_result [catch {exec $BINARIES/mris_make_surfaces -q $subjname rh  >&@ $logfileId} rh_msg]
		    puts $logfileId ""
		}

	    }

	    2 {

		# check for existence of inflated surface
		if { ![file exists $SUBJECTS_DIR/$subjname/surf/lh.inflated] } {
		    error "MakeFinalSurfaces: can't find lh.inflated surface file"
		}

		if {$VERBOSE_FLAG} {
		    puts stdout ""
		    puts stdout "Reconstructing the final left hemisphere surfaces . . ."
		    set lh_result [catch {exec $BINARIES/mris_make_surfaces -q $subjname lh >&@ stdout} lh_msg]
		    puts stdout ""
		} else {
		    puts $logfileId ""
		    puts $logfileId "Reconstructing the final left hemisphere surfaces . . ."
		    set lh_result [catch {exec $BINARIES/mris_make_surfaces -q $subjname lh >&@ logfileId} lh_msg]
		    puts $logfileId ""
		}

	    }

	    default {

		# check for existence of inflated surface
		if { ![file exists $SUBJECTS_DIR/$subjname/surf/rh.inflated] } {
		    error "MakeFinalSurfaces: can't find rh.inflated surface file"
		}

		if {$VERBOSE_FLAG} {
		    puts stdout ""
		    puts stdout "Reconstructing the final right hemisphere surfaces . . ."
		    set rh_result [catch {exec $BINARIES/mris_make_surfaces -q $subjname rh >&@ stdout} rh_msg]
		    puts stdout ""
		} else {
		    puts $logfileId ""
		    puts $logfileId "Reconstructing the final right hemisphere surfaces . . ."
		    set rh_result [catch {exec $BINARIES/mris_make_surfaces -q $subjname rh >&@ $logfileId} rh_msg]
		    puts $logfileId ""
		}

	    }

	}

    # get the end time for processing
    set EndTime [exec date]

    puts stdout ". . . ending at $EndTime . . ."
    puts $logfileId ". . . ending at $EndTime . . ."
    puts ""

    # don't forget to close the logfile!
    close $logfileId

    return 0
}

# this procedure flattens patches and whole hemispheres
# the assumptions are that the patches and hemispheres to
# be flattened live in $SUBJECTS_DIR/$subjname/surf and
# conform to the following naming convention:
#
# hemi.*patch.0 - occipital patch with relaxation cuts
# hemi.*nomid.0 - whole hemisphere with relaxation cuts and medial portion removed
#
# flattened surfaces will be output with the original filename and 1000 appended to
# the end so:
# 
# hemi.*patch.0 -> hemi.*patch.1000
# hemi.*nomid.0 -> hemi.*nomid.1000
#
proc FlattenSurfaces { {WhichHemisphere 1} } {

    global BINARIES
    global subjname
    global SUBJECTS_DIR
    global VERBOSE_FLAG
    global AFNIBIN
    global env
	
    set processdir [concat $SUBJECTS_DIR/$subjname]

    if [catch {open $processdir/MakeSurf.log a+ 0666} logfileId] {
	error "FlattenSurfaces: Cannot open $processdir/MakeSurf.log: $logfileId"
    }
	
    # go to the scripts dir
    cd $SUBJECTS_DIR/$subjname/scripts

    switch -exact -- $WhichHemisphere {
	
	    1 {

		# get list of rh patch and nomid files
		puts stdout ""
		puts stdout "The surf directory contains the following patch and nomid files:"
		puts stdout ""

		# patches . . . 
		set patchfiles [glob -nocomplain $SUBJECTS_DIR/$subjname/surf/rh*patch*.0]

		foreach patch $patchfiles {
		    puts stdout $patch
		}
		

		# nomid . . . 
		set nomidfiles [glob -nocomplain $SUBJECTS_DIR/$subjname/surf/rh*nomid*.0]

		foreach nomid $nomidfiles {
		    puts stdout $nomid
		}
		
		if { [llength $patchfiles] == 0 && [llength $nomidfiles] == 0} {
		    error "FlattenSurfaces: There are no rh patch or nomid files in $SUBJECTS_DIR/$subjname/surf"
		}

		puts stdout ""
		puts stdout "How many surfaces do you want to flatten?"
		puts -nonewline stdout {: }
		flush stdout  ;#
		set numtoflatten [gets stdin]

		if { $numtoflatten != [expr [expr [llength $patchfiles]] + [expr [llength $nomidfiles]]] } {	
			error "FlattenSurfaces: Directory $SUBJECTS_DIR/$subjname/surf only contains [expr [expr [llength $patchfiles]] + [expr [llength $nomidfiles]]] surface files!"	
		}
		

		# make list of files to be flattened
		set i 1
		while { $i <= $numtoflatten } {

		    puts stdout ""
		    puts stdout "Enter surface number $i to be flattened"
		    puts -nonewline stdout {: }
		    flush stdout  ;#
		    set tmpfile [gets stdin]

		    
		    # check for existence of inflated surface
		    if { ![file exists $tmpfile] } {
		        error "FlattenSurfaces: $tmpfile does not exist"
		    }

		    lappend surfacelist $tmpfile

		    unset tmpfile
		    
		    incr i 
		}

	
		foreach surface $surfacelist {
		    
		    if {$VERBOSE_FLAG} {
		
			puts stdout ""
			puts stdout "Flattening right hemisphere surface: [file tail $surface]"
			puts stdout ""

			# get output file prefix
			scan [file tail $surface] {%[a-z.]} outfile
			

			# mris_flatten called differently for occipital patch vs whole hemisphere
			if { [regexp -nocase nomid $outfile] } {
			    puts stdout "$BINARIES/mris_flatten -w 0 [format ../surf/%s [file tail $surface]] [format ../surf/%s%d  $outfile 1000]"
			    set rhflat_result [catch {exec $BINARIES/mris_flatten -w 0 [format ../surf/%s [file tail $surface]] [format ../surf/%s%d $outfile 1000]  >&@ stdout} rhflat_msg]
			} else {
			    puts stdout "$BINARIES/mris_flatten -w 0 -distances 20 10 [format ../surf/%s [file tail $surface]] [format ../surf/%s%d $outfile 1000]"
			    set rhflat_result [catch {exec $BINARIES/mris_flatten -w 0 -distances 20 10 [format ../surf/%s [file tail $surface]] [format ../surf/%s%d $outfile 1000]  >&@ stdout} rhflat_msg]
			}
			puts stdout ""
		    } else {

			puts $logfileId ""
			puts $logfileId "Flattening right hemisphere surface: [file tail $surface]"
			puts $logfileId ""

			# get output file prefix
			scan [file tail $surface] {%[a-z.]} outfile

			# mris_flatten called differently for occipital patch vs whole hemisphere
			if { [regexp -nocase nomid $outfile] } {
			    puts stdout "$BINARIES/mris_flatten -w 0 [format ../surf/%s [file tail $surface]] [format ../surf/%s%d $outfile 1000]"
			    set rhflat_result [catch {exec $BINARIES/mris_flatten -w 0 [format ../surf/%s [file tail $surface]] [format ../surf/%s%d $outfile 1000] >&@ $logfile} rhflat_msg]
			} else {
			    puts stdout "$BINARIES/mris_flatten -w 0 -distances 20 10 [format ../surf/%s [file tail $surface]] [format ../surf/%s%d $outfile 1000]"
			    set rhflat_result [catch {exec $BINARIES/mris_flatten -w 0 -distances 20 10 [format ../surf/%s [file tail $surface]] [format ../surf/%s%d $outfile 1000] >&@ $logfile} rhflat_msg]
			}
			
			puts $logfileId ""
		    }

		    
		}



	    }

	    2 {

		
		# get list of lh patch and nomid files
		puts stdout ""
		puts stdout "The surf directory contains the following patch and nomid files:"
		puts stdout ""

		# patches . . . 
		set patchfiles [glob -nocomplain $SUBJECTS_DIR/$subjname/surf/lh*patch*.0]

		foreach patch $patchfiles {
		    puts stdout $patch
		}
		

		# nomid . . . 
		set nomidfiles [glob -nocomplain $SUBJECTS_DIR/$subjname/surf/lh*nomid*.0]

		foreach nomid $nomidfiles {
		    puts stdout $nomid
		}
		
		if { [llength $patchfiles] == 0 && [llength $nomidfiles] == 0} {
		    error "FlattenSurfaces: There are no lh patch or nomid files in $SUBJECTS_DIR/$subjname/surf"
		}

		
		puts stdout ""
		puts stdout "How many surfaces do you want to flatten?"
		puts -nonewline stdout {: }
		flush stdout  ;#
		set numtoflatten [gets stdin]

		if { $numtoflatten != [expr [expr [llength $patchfiles]] + [expr [llength $nomidfiles]]] } {	
			error "FlattenSurfaces: Directory $SUBJECTS_DIR/$subjname/surf only contains [expr [expr [llength $patchfiles]] + [expr [llength $nomidfiles]]] surface files!"	
		}
		
		

		# make list of files to be flattened
		set i 1
		while { $i <= $numtoflatten } {

		    puts stdout ""
		    puts stdout "Enter surface number $i to be flattened"
		    puts -nonewline stdout {: }
		    flush stdout  ;#
		    set tmpfile [gets stdin]

		    # check for existence of inflated surface
		    if { ![file exists $tmpfile] } {
		        error "FlattenSurfaces: $tmpfile does not exist"
		    }

		    lappend surfacelist $tmpfile

		    unset tmpfile
		    
		    incr i
		}

		
		foreach surface $surfacelist {

		    if {$VERBOSE_FLAG} {

			puts stdout ""
			puts stdout "Flattening left hemisphere surface: [file tail $surface]"

			# get output file prefix
			scan [file tail $surface] {%[a-z.]} outfile

			# mris_flatten called differently for occipital patch vs whole hemisphere
			if { [regexp -nocase nomid $outfile] } {
			    puts stdout "$BINARIES/mris_flatten -w 0 [format ../surf/%s [file tail $surface]] [format ../surf/%s%d $outfile 1000]"
			    set lhflat_result [catch {exec $BINARIES/mris_flatten -w 0 [format ../surf/%s [file tail $surface]] [format ../surf/%s%d $outfile 1000]  >&@ stdout} lhflat_msg]
			} else {
			    puts stdout "$BINARIES/mris_flatten -w 0 -distances 20 10 [format ../surf/%s [file tail $surface]] [format ../surf/%s%d $outfile 1000]"
			    set lhflat_result [catch {exec $BINARIES/mris_flatten -w 0 -distances 20 10 [format ../surf/%s [file tail $surface]] [format ../surf/%s%d $outfile 1000]  >&@ stdout} lhflat_msg]
			}
			puts stdout ""
		    } else {

			puts $logfileId ""
			puts $logfileId "Flattening left hemisphere surface: [file tail $surface]"

			# get output file prefix
			scan $surface {%[a-z.]} outfile

			# mris_flatten called differently for occipital patch vs whole hemisphere
			if { [regexp -nocase nomid $outfile] } {
			    puts stdout "$BINARIES/mris_flatten -w 0 [format ../surf/%s [file tail $surface]] [format ../surf/%s%d $outfile 1000]"
			    set lhflat_result [catch {exec $BINARIES/mris_flatten -w 0 [format ../surf/%s [file tail $surface]] [format ../surf/%s%d $outfile 1000] >&@ $logfile} lhflat_msg]
			} else {
			    puts stdout "$BINARIES/mris_flatten -w 0 -distances 20 10 [format ../surf/%s [file tail $surface]] [format ../surf/%s%d $outfile 1000]"
			    set lhflat_result [catch {exec $BINARIES/mris_flatten -w 0 -distances 20 10 [format ../surf/%s [file tail $surface]] [format ../surf/%s%d $outfile 1000] >&@ $logfile} lhflat_msg]
			}
			
			puts $logfileId ""
		    }

		    
		}
		
	    }

	    default {

		# get list of rh patch and nomid files
		puts stdout ""
		puts stdout "The surf directory contains the following patch and nomid files:"
		puts stdout ""

		# patches . . . 
		set patchfiles [glob -nocomplain $SUBJECTS_DIR/$subjname/surf/rh*patch*.0]

		foreach patch $patchfiles {
		    puts stdout $patch
		}
		

		# nomid . . . 
		set nomidfiles [glob -nocomplain $SUBJECTS_DIR/$subjname/surf/rh*nomid*.0]

		foreach nomid $nomidfiles {
		    puts stdout $nomid
		}
		
		if { [llength $patchfiles] == 0 && [llength $nomidfiles] == 0} {
		    error "FlattenSurfaces: There are no rh patch or nomid files in $SUBJECTS_DIR/$subjname/surf"
		}

		puts stdout ""
		puts stdout "How many surfaces do you want to flatten?"
		puts -nonewline stdout {: }
		flush stdout  ;#
		set numtoflatten [gets stdin]

		if { $numtoflatten != [expr [expr [llength $patchfiles]] + [expr [llength $nomidfiles]]] } {	
			error "FlattenSurfaces: Directory $SUBJECTS_DIR/$subjname/surf only contains [expr [expr [llength $patchfiles]] + [expr [llength $nomidfiles]]] surface files!"	
		}
		

		# make list of files to be flattened
		set i 1
		while { $i <= $numtoflatten } {

		    puts stdout ""
		    puts stdout "Enter surface number $i to be flattened"
		    puts -nonewline stdout {: }
		    flush stdout  ;#
		    set tmpfile [gets stdin]

		    # check for existence of inflated surface
		    if { ![file exists $tmpfile] } {
		        error "FlattenSurfaces: $tmpfile does not exist"
		    }

		    lappend surfacelist $tmpfile

		    unset tmpfile
		    
		    incr i
		}

		
		foreach surface $surfacelist {

		    if {$VERBOSE_FLAG} {

			puts stdout ""
			puts stdout "Flattening right hemisphere surface: [file tail $surface]"

			# get output file prefix
			scan [file tail $surface] {%[a-z.]} outfile
			

			# mris_flatten called differently for occipital patch vs whole hemisphere
			if { [regexp -nocase nomid $outfile] } {
			    puts stdout "$BINARIES/mris_flatten -w 0 [format ../surf/%s [file tail $surface]] [format ../surf/%s%d $outfile 1000]"
			    set rhflat_result [catch {exec $BINARIES/mris_flatten -w 0 [format ../surf/%s [file tail $surface]] [format ../surf/%s%d $outfile 1000]  >&@ stdout} rhflat_msg]
			} else {
			    puts stdout "$BINARIES/mris_flatten -w 0 -distances 20 10 [format ../surf/%s [file tail $surface]] [format ../surf/%s%d $outfile 1000]"
			    set rhflat_result [catch {exec $BINARIES/mris_flatten -w 0 -distances 20 10 [format ../surf/%s [file tail $surface]] [format ../surf/%s%d $outfile 1000]  >&@ stdout} rhflat_msg]
			}
			puts stdout ""
		    } else {

			puts $logfileId ""
			puts $logfileId "Flattening right hemisphere surface: [file tail $surface]"

			# get output file prefix
			scan [file tail $surface] {%[a-z.]} outfile
			

			# mris_flatten called differently for occipital patch vs whole hemisphere
			if { [regexp -nocase nomid $outfile] } {
			    puts stdout "$BINARIES/mris_flatten -w 0 [format ../surf/%s [file tail $surface]] [format ../surf/%s%d $outfile 1000]"
			    set rhflat_result [catch {exec $BINARIES/mris_flatten -w 0 [format ../surf/%s [file tail $surface]] [format ../surf/%s%d $outfile 1000] >&@ $logfile} rhflat_msg]
			} else {
			    puts stdout "$BINARIES/mris_flatten -w 0 -distances 20 10 [format ../surf/%s [file tail $surface]] [format ../surf/%s%d $outfile 1000]"
			    set rhflat_result [catch {exec $BINARIES/mris_flatten -w 0 -distances 20 10 [format ../surf/%s [file tail $surface]] [format ../surf/%s%d $outfile 1000] >&@ $logfile} rhflat_msg]
			}
			
			puts $logfileId ""
		    }

		    
		}

	    }

	}


    # don't forget to close the logfile!
    close $logfileId

    return 0
}


proc DoRegister { } {

    global BINARIES
    global BINARIES2
    global subjname
    global SUBJECTS_DIR
    global VERBOSE_FLAG
    global AFNIBIN
    global env
	
    set processdir [concat $SUBJECTS_DIR/$subjname]

    if [catch {open $processdir/MakeSurf.log a+ 0666} logfileId] {
	error "DoRegister: Cannot open $processdir/MakeSurf.log: $logfileId"
    }
	
    # register needs the COR files in the T1 directory
    # make check to see if they are there, if not exit
    # there should be 257 file, 256 image files plus the info file
    set T1files [glob -nocomplain $SUBJECTS_DIR/$subjname/mri/T1/COR-.*]
    if { [llength $T1files] != 257 } {
	#error "DoRegister: Insufficient number of T1 COR files to do registration"
    }
    # now check the file sizes of each, each image COR  should be 65536 bytes 
    set corindex 1
    foreach file $T1files {
	if { $corindex > 1 } {
	    set x [file size $file]
	    if { $x != [expr 256*256] } {
		#error "DoRegister: Improper file size for $file"
	    }
	}
	incr corindex
    }
    

    # Assumption: the functional data exist in a directory separate from the 
    # surface directory. For now prompt the user to enter this functional directory

    puts stdout ""
    puts stdout "Enter the pathname of the functional directory"
    puts -nonewline stdout {: }
    flush stdout  ;#
    set funcdir [gets stdin]

    # check its existence
    if { ![file isdirectory $funcdir] } {
	puts stdout ""
	puts stdout "$funcdir does not exist, so it will be created"
	puts stdout ""
	exec mkdir $funcdir
	#error "DoRegister: Functional directory, $funcdir, does not exist!"
    }

    cd $funcdir

    # check for existence of analyse.dat and register.dat

    # if register.dat exists but analyse.dat does not then remove register.dat and prompt for new info
    if { ![file exists $funcdir/analyse.dat] && [file exists $funcdir/register.dat] } {
	exec rm $funcdir/register.dat
    }

    # if analyse.dat exists but register.dat does not then remove analyse.dat and prompt for new info
    if { [file exists $funcdir/analyse.dat] && ![file exists $funcdir/register.dat] } {
	exec rm $funcdir/analyse.dat
    }

    # initialize registration method variable 
    set registerchoice 2
    # if both analyse.dat and register.dat exist then show contents and present options
    if { [file exists $funcdir/analyse.dat] && [file exists $funcdir/register.dat] } {
	puts stdout ""
	puts stdout "The directory you specified, $funcdir,"
	puts stdout "contains information from a previous registration:"
	puts stdout ""
	# display contents of register.dat and analyse.dat
	ReadAnalyseDat $funcdir/analyse.dat
	ReadRegisterDat $funcdir/register.dat
	puts stdout ""
	puts stdout "You have the following options:"
	puts stdout "1: Use the current registration information"
	puts stdout "2: Start over"

	puts stdout ""
	puts -nonewline stdout "Choose (1 or 2): "
	flush stdout  ;#
	set registerchoice [gets stdin]

	if { $registerchoice == 2 } {
	    exec rm $funcdir/analyse.dat
	    exec rm $funcdir/register.dat
	}
    }

    # if both analyse.dat and register.dat do not exist then prompt for new info

    if { $registerchoice == 2 } {

	# get the name of an epi or coplanar structural image file to be converted to bshort format
	puts stdout ""
	puts stdout "Enter the name (relative or absolute path) of an ANALYZE or AFNI"
	puts stdout "format EPI or coplanar structural image that will be used in "
	puts stdout "registering the functional data to the high-resolution volume"
	puts -nonewline stdout {: }
	flush stdout  ;#
	set imgfilename [gets stdin]

	# identify whether it is an ANALYZE or AFNI format file
	# the decision criteria are:
	# - if it ends in .hdr or .img, then it is an ANALYZE format image file
	# - if it ends in .HEAD or .BRIK then it is an AFNI format image file
	# after the file has been removed the tail is truncated and it is converted
	# to bshort MGH format for use with the register program

	
	if { [string match *.img [file tail $imgfilename]] } {
	    set filetype ANALYZE
	}
	if { [string match *.hdr [file tail $imgfilename]] } {
	    set filetype ANALYZE
	}
	if { [string match *.HEAD [file tail $imgfilename]] } {
	    set filetype AFNI
	}
	if { [string match *.BRIK [file tail $imgfilename]] } {
	    set filetype AFNI	
	}

	if { ![info exists filetype] } {
	    error "DoRegister: can't determine filetype for: $imgfilename"
	}

	

	switch -exact -- $filetype {
	
		
		ANALYZE {

		    puts stdout ""
		    puts stdout "Processing ANALYZE filetype"

		    set img_pathname [format %s%s [file rootname $imgfilename] .img]   
		    set hdr_pathname [format %s%s [file rootname $imgfilename] .hdr]

		    puts stdout "ANALYZE img name: $img_pathname"
		    puts stdout "ANALYZE hdr name: $hdr_pathname"

		    # check for existence of img and hdr names
		    if { ![file exists $img_pathname] } {
			error "DoRegister: can't find $img_pathname"
		    }
		    if { ![file exists $hdr_pathname] } {
			error "DoRegister: can't find $hdr_pathname"
		    }

		    # read ANALYZE header information
		    set hdr_props [exec $BINARIES2/GetAVWInfo $hdr_pathname]
		    set num_props [llength $hdr_props]
	
		    if {$num_props != 14} {	
			error "Cannot read header name: $hdr_pathname"	
		    }
	
		    set xdim [lindex $hdr_props 0]
		    set ydim [lindex $hdr_props 1]
		    set zdim [lindex $hdr_props 2]
		    set xvoxdim [lindex $hdr_props 3]
		    set yvoxdim [lindex $hdr_props 4]
		    set zvoxdim [lindex $hdr_props 5]
		    set bits [lindex $hdr_props 7]
		    set max [lindex $hdr_props 8]
		    set min [lindex $hdr_props 9]
		    set datatypecode [lindex $hdr_props 12]
	
		    set slicethickness $zvoxdim
		    set FOV [expr $xdim * $xvoxdim]
	
		    puts stdout ""
		    puts stdout "ANALYZE header contents for:"
		    puts stdout "$hdr_pathname"
		    puts stdout "X,Y,Z dimensions: $xdim $ydim $zdim"
		    puts stdout "X,Y,Z voxel sizes: $xvoxdim $yvoxdim $zvoxdim"
		    puts stdout "Maximum = $max, Minimum = $min"
		    puts stdout "Pixel encoding: $bits bits"
		    puts stdout "AVW datatypecode: $datatypecode"
		    puts stdout ""
    	
		    puts $logfileId ""
		    puts $logfileId "ANALYZE header contents for:"
		    puts $logfileId "$hdr_pathname"
		    puts $logfileId "X,Y,Z dimensions: $xdim $ydim $zdim"
		    puts $logfileId "X,Y,Z voxel sizes: $xvoxdim $yvoxdim $zvoxdim"
		    puts $logfileId "Maximum = $max, Minimum = $min"
		    puts $logfileId "Pixel encoding: $bits bits"
		    puts $logfileId "AVW datatypecode: $datatypecode"
		    puts $logfileId ""

		    #
		    # AVW to MGH bshort conversion
		    #

		    # analyze2bshort is called with the following arguments:
		    # Usage: analyze2bshort [-options] -i <instem> -o <outstem> -ntime <ntime>
		    #  -i <instem>     : prefix of Analyze .img input file
		    #  -o <outstem>    : prefix of .bfloat output file
		    #  -ntime <ntime>  : number of timepoints
		    #  -first <num>    : first time point

		    # of course analyze2bshort is very picky about the input file naming convention
		    # if my input file is named "mean7.hdr" it will expect "mean7001.hdr" so this 
		    # problem is solved here temporarily with symbolic linking

		    set linktohdr [format %s0001.hdr [file rootname $hdr_pathname]]
		    #puts stdout "here1: $linktohdr"
		    if { [file exists $linktohdr] } {
			error "DoRegister: link already exists between $hdr_pathname and $linktohdr"
		    }
		    puts stdout "linking: $hdr_pathname to [format %s0001.hdr [file rootname $hdr_pathname]]"
		    set link_result [catch {exec ln -s $hdr_pathname $linktohdr} link_msg]
		    
		    # check to see if the link has been made successfully
		    if { ![file exists $linktohdr] } {
			error "DoRegister: failed to create link to $linktohdr"
		    }

		    set linktoimg [format %s0001.img [file rootname $img_pathname]]
		    #puts stdout "here2: $linktoimg"
		    if { [file exists $linktoimg] } {
			error "DoRegister: link already exists between $img_pathname and $linktoimg"
		    }
		    puts stdout "linking: $img_pathname to [format %s0001.img [file rootname $img_pathname]]"
		    set link_result [catch {exec ln -s $img_pathname $linktoimg} link_msg]

		    # check to see if the link has been made successfully
		    if { ![file exists $linktoimg] } {
			error "DoRegister: failed to create link to $linktoimg"
		    }
  
		    # finally we can call analyze2bshort
		    set stemname [file rootname $imgfilename]
		    set avw2bshort_result [catch {exec $BINARIES2/analyze2bshort -i $stemname -o $stemname -ntime 1 -first 1 >&@ stdout} avw2bshort_msg]

		    

	            # check to see if bshorts have been made correctly

		    # remove the symbolic links 
		    set rm_result [catch {exec rm -rf $linktoimg} rm_msg]
		    set rm_result [catch {exec rm -rf $linktohdr} rm_msg]

		    # its safe to open the analyse.dat file for writing
		    if [catch {open $funcdir/analyse.dat w} analyse_dat_fid] {
			error "DoRegister: Can't open for writing: $funcdir/analyse.dat"
		    }

		    # fill analyse.dat file with info
		    # an example of the contents
		    #.
		    #AHt1r1_01_%03d.bshort
		    #22 1
		    #64 64
		    # "." indicates bshorts are in current directory
		    puts $analyse_dat_fid "."
		    # bshort name goes here
		    set stem1 [format %s_ $stemname]
		    set stem2 "%03d"
		    set stem3 ".bshort"
		    puts $analyse_dat_fid "$stem1$stem2$stem3"
		    # zdim and nperslice, which is the number of reps or volumes 
		    # in this case it should really always be one
		    puts $analyse_dat_fid "$zdim 1"
		    # xdim followed by ydim
		    puts $analyse_dat_fid "$xdim $ydim"
		    close $analyse_dat_fid

		    # create the register matrix
		    MakeRegisterMatrix $funcdir/register.dat $xvoxdim $zvoxdim

		}

		AFNI {
		
		    puts stdout ""
		    puts stdout "Processing AFNI filetype"

		    set brik_pathname [format %s%s [file rootname $imgfilename] .BRIK]   
		    set head_pathname [format %s%s [file rootname $imgfilename] .HEAD]
		    
		    set stemname [file rootname $imgfilename]

		    puts stdout "AFNI BRIK name: $brik_pathname"
		    puts stdout "AFNI HEAD name: $head_pathname"

		    # check for existence of brik and head names
		    if { ![file exists $brik_pathname] } {
			error "DoRegister: can't find $brik_pathname"
		    }
		    if { ![file exists $head_pathname] } {
			error "DoRegister: can't find $head_pathname"
		    }

		    

		    # call ReadAFNIHEAD proc
		    set HEADInfo [ReadAFNIHEAD $head_pathname]

		    set afni_xdim [lindex $HEADInfo 0]
		    set afni_ydim [lindex $HEADInfo 1]
		    set afni_zdim [lindex $HEADInfo 2]
		    set afni_xvox [lindex $HEADInfo 3]
		    set afni_yvox [lindex $HEADInfo 4]
		    set afni_zvox [lindex $HEADInfo 5]
		    set afni_briksize [file size $brik_pathname]
		    set afni_planesize [expr $afni_briksize / $afni_zdim]

		    puts stdout ""
		    puts stdout "AFNI HEAD Information:"
		    puts stdout "X Dimension: $afni_xdim"
		    puts stdout "Y Dimension: $afni_ydim"
		    puts stdout "Z Dimension: $afni_zdim"
		    puts stdout "X Voxel Size: $afni_xvox"
		    puts stdout "Y Voxel Size: $afni_yvox"
		    puts stdout "Z Voxel Size: $afni_zvox"
		    puts stdout "Byte-size of Whole BRIK: $afni_briksize"
		    puts stdout "Byte-size of One 2D Slice: $afni_planesize"
		    puts stdout ""

		    puts stdout "Creating bshort files from $brik_pathname . . . "
		    puts stdout ""
		    # can use the following to split
		    # -b codes the bytesize, -a means append only one letter after "tmp" for output files
		    set split_result [catch {exec split -b $afni_planesize -a 1 $brik_pathname tmp >&@ stdout} split_msg]

		    # rename output
		    # output will be in the current directory and will be tmp[a-z] if afni_zdim <= 26

		    if { $afni_zdim > 26} {
			error "DoRegister: $brik_pathname has more than 26 slices, can't format split output."
		    } else {

			set letternum 1
			foreach letter {a b c d e f g h i j k l m n o p q r s t u v w x y z} {
			    
			    if { $letternum > $afni_zdim } {
				break
			    }

			    set outputblock [format tmp%s $letter]
			    set briktail [file tail $brik_pathname]
			    set bshortblock [format %s_%03d.bshort [file rootname $briktail] [expr $letternum - 1]]
			    set bshorthdr [format %s_%03d.hdr [file rootname $briktail] [expr $letternum - 1]]
			    exec mv $outputblock $bshortblock

			    # open hdr file
			    if [catch {open $funcdir/$bshorthdr w} bshorthdr_fid] {
				error "DoRegister: Can't open for writing: $funcdir/$bshorthdr"
			    }
			    # fill with bshort image plane information
			    puts $bshorthdr_fid "$afni_xdim $afni_ydim 1 0"
			    # close hdr file
			    close $bshorthdr_fid 
			    incr letternum
			}
		    }

		    # its safe to open the analyse.dat file for writing
		    if [catch {open $funcdir/analyse.dat w} analyse_dat_fid] {
			error "DoRegister: Can't open for writing: $funcdir/analyse.dat"
		    }

		    # fill analyse.dat file with info
		    # an example of the contents
		    #.
		    #AHt1r1_01_%03d.bshort
		    #22 1
		    #64 64
		    # "." indicates bshorts are in current directory
		    puts $analyse_dat_fid "."
		    # bshort name goes here
		    set stem1 [format %s_ $stemname]
		    set stem2 "%03d"
		    set stem3 ".bshort"
		    puts $analyse_dat_fid "$stem1$stem2$stem3"
		    # zdim and nperslice, which is the number of reps or volumes 
		    # in this case it should really always be one
		    puts $analyse_dat_fid "$afni_zdim 1"
		    # xdim followed by ydim
		    puts $analyse_dat_fid "$afni_xdim $afni_ydim"
		    close $analyse_dat_fid

		    # create the register matrix
		    MakeRegisterMatrix $funcdir/register.dat $afni_xvox $afni_zvox

		    

		}
		
		default {

		    error "Unidentified filetype: $filetype"
		
		
		}
		
	    }

    

    }

    # a statistical map may be used to view the results of the registration
    # let user input a floating point ANALYZE volume or AFNI BRIK

    # get the name of the floating point statistical map
    puts stdout ""
    puts stdout "A 32-bit ANALYZE or AFNI statistical volume may be used"
    puts stdout "as an overlay to view the results of the registration."
    puts stdout "The statistical volume must have the same dimensions as"
    puts stdout "the echo-planar image used in the alignment."
    puts stdout ""
    puts stdout "Enter name of a floating point statistical map"
    puts -nonewline stdout {: }
    flush stdout  ;#
    set imgfilename [gets stdin]

    puts stdout ""
    puts stdout "Enter a threshold to use for the statistical overlay:"
    puts -nonewline stdout {: }
    flush stdout  ;#
    set fthresh [gets stdin]

    # identify whether it is an ANALYZE or AFNI format file
    # the decision criteria are:
    # - if it ends in .hdr or .img, then it is an ANALYZE format image file
    # - if it ends in .HEAD or .BRIK then it is an AFNI format image file
    # after the file has been removed the tail is truncated and it is converted
    # to bshort MGH format for use with the register program

	
    if { [string match *.img [file tail $imgfilename]] } {
	set filetype ANALYZE
    }
    if { [string match *.hdr [file tail $imgfilename]] } {
	set filetype ANALYZE
    }
    if { [string match *.HEAD [file tail $imgfilename]] } {
	set filetype AFNI
    }
    if { [string match *.BRIK [file tail $imgfilename]] } {
	set filetype AFNI	
    }

    if { ![info exists filetype] } {
	error "DoRegister: can't determine filetype for: $imgfilename"
    }

    switch -exact -- $filetype {
	
		
	ANALYZE {

	    puts stdout ""
	    puts stdout "Processing ANALYZE filetype"

	    set img_pathname [format %s%s [file rootname $imgfilename] .img]   
	    set hdr_pathname [format %s%s [file rootname $imgfilename] .hdr]

	    puts stdout "ANALYZE img name: $img_pathname"
	    puts stdout "ANALYZE hdr name: $hdr_pathname"

	    # read ANALYZE header information
	    set hdr_props [exec $BINARIES2/GetAVWInfo $hdr_pathname]
	    set num_props [llength $hdr_props]
	
	    if {$num_props != 14} {	
		error "DoRegister: Cannot read header name: $hdr_pathname"	
	    }
	
	    set xdim [lindex $hdr_props 0]
	    set ydim [lindex $hdr_props 1]
	    set zdim [lindex $hdr_props 2]
	    set xvoxdim [lindex $hdr_props 3]
	    set yvoxdim [lindex $hdr_props 4]
	    set zvoxdim [lindex $hdr_props 5]
	    set bits [lindex $hdr_props 7]
	    set max [lindex $hdr_props 8]
	    set min [lindex $hdr_props 9]
	    set datatypecode [lindex $hdr_props 12]
	
	    set slicethickness $zvoxdim
	    set FOV [expr $xdim * $xvoxdim]
	
	    puts stdout ""
	    puts stdout "ANALYZE header contents for:"
	    puts stdout "$hdr_pathname"
	    puts stdout "X,Y,Z dimensions: $xdim $ydim $zdim"
	    puts stdout "X,Y,Z voxel sizes: $xvoxdim $yvoxdim $zvoxdim"
	    puts stdout "Maximum = $max, Minimum = $min"
	    puts stdout "Pixel encoding: $bits bits"
	    puts stdout "AVW datatypecode: $datatypecode"
	    puts stdout ""
    	
	    puts $logfileId ""
	    puts $logfileId "ANALYZE header contents for:"
	    puts $logfileId "$hdr_pathname"
	    puts $logfileId "X,Y,Z dimensions: $xdim $ydim $zdim"
	    puts $logfileId "X,Y,Z voxel sizes: $xvoxdim $yvoxdim $zvoxdim"
	    puts $logfileId "Maximum = $max, Minimum = $min"
	    puts $logfileId "Pixel encoding: $bits bits"
	    puts $logfileId "AVW datatypecode: $datatypecode"
	    puts $logfileId ""

	    # convert to MGH bfloat format
 
	    # finally we can call analyze2mgh
	    set stemname [file rootname $imgfilename]

	    puts stdout ""
	    puts stdout "Stemname: $stemname"
	    puts stdout ""

	    set avw2mgh_result [catch {exec $BINARIES2/analyze2mgh -i $stemname >&@ stdout} avw2mgh_msg]

	    # check to see if bfloats have been made correctly

	    # bfloat name goes here
	    set stem1 [format %s_ $stemname]
	    set stem2 "%03d"
	    set stem3 ".bfloat"
	    set regstatinput $stem1$stem2$stem3
		    
	    puts stdout ""
	    puts stdout "register statistical map input name: $regstatinput"
		   
	}

	AFNI {
		
	    puts stdout ""
	    puts stdout "Processing AFNI filetype"

	    set brik_pathname [format %s%s [file rootname $imgfilename] .BRIK]   
	    set head_pathname [format %s%s [file rootname $imgfilename] .HEAD]
		    
	    set stemname [file rootname $imgfilename]

	    puts stdout "AFNI BRIK name: $brik_pathname"
	    puts stdout "AFNI HEAD name: $head_pathname"

	    # check for existence of brik and head names
	    if { ![file exists $brik_pathname] } {
		error "DoRegister: can't find $brik_pathname"
	    }
	    if { ![file exists $head_pathname] } {
		error "DoRegister: can't find $head_pathname"
	    }

		   
	    # call ReadAFNIHEAD proc
	    set HEADInfo [ReadAFNIHEAD $head_pathname]

	    set afni_xdim [lindex $HEADInfo 0]
	    set afni_ydim [lindex $HEADInfo 1]
	    set afni_zdim [lindex $HEADInfo 2]
	    set afni_xvox [lindex $HEADInfo 3]
	    set afni_yvox [lindex $HEADInfo 4]
	    set afni_zvox [lindex $HEADInfo 5]
	    set afni_briksize [file size $brik_pathname]
	    set afni_planesize [expr $afni_briksize / $afni_zdim]

	    puts stdout ""
	    puts stdout "AFNI HEAD Information:"
	    puts stdout "X Dimension: $afni_xdim"
	    puts stdout "Y Dimension: $afni_ydim"
	    puts stdout "Z Dimension: $afni_zdim"
	    puts stdout "X Voxel Size: $afni_xvox"
	    puts stdout "Y Voxel Size: $afni_yvox"
	    puts stdout "Z Voxel Size: $afni_zvox"
	    puts stdout "Byte-size of Whole BRIK: $afni_briksize"
	    puts stdout "Byte-size of One 2D Slice: $afni_planesize"
	    puts stdout ""

	    puts stdout "Creating bfloat files from $brik_pathname . . . "
	    puts stdout ""
	    # can use the following to split
	    # -b codes the bytesize, -a means append only one letter after "tmp" for output files
	    set split_result [catch {exec split -b $afni_planesize -a 1 $brik_pathname tmp >&@ stdout} split_msg]

	    # rename output
	    # output will be in the current directory and will be tmp[a-z] if afni_zdim <= 26

	    if { $afni_zdim > 26} {
		error "DoRegister: $brik_pathname has more than 26 slices, can't format split output."
	    } else {

		set letternum 1
		foreach letter {a b c d e f g h i j k l m n o p q r s t u v w x y z} {
			    
		    if { $letternum > $afni_zdim } {
			break
		    }

		    set outputblock [format tmp%s $letter]
		    set briktail [file tail $brik_pathname]
		    set bfloatblock [format %s_%03d.bfloat [file rootname $briktail] [expr $letternum - 1]]
		    set bfloathdr [format %s_%03d.hdr [file rootname $briktail] [expr $letternum - 1]]
		    exec mv $outputblock $bfloatblock

		    # open hdr file
		    if [catch {open $funcdir/$bfloathdr w} bfloathdr_fid] {
			error "DoRegister: Can't open for writing: $funcdir/$bfloathdr"
		    }
		    # fill with bfloat image plane information
		    puts $bfloathdr_fid "$afni_xdim $afni_ydim 1 0"
		    # close hdr file
		    close $bfloathdr_fid 
		    incr letternum
		}
	    }

	    # bfloat name goes here
	    set stem1 [format %s_ $stemname]
	    set stem2 "%03d"
	    set stem3 ".bfloat"
	    set regstatinput $stem1$stem2$stem3
		    
	    puts stdout ""
	    puts stdout "register statistical map input name: $regstatinput"
		    
	    

	}
		
	default {

	    error "Unidentified filetype: $filetype"	
		
	}
		
    }

    # here's how scanair can be used to get the contents of a *.air file
    # set contents [exec /apps/medx3/bin/air/scanair_16u xformfirst.air -r]
    
    
    # get the start time for processing
    set StartTime [exec date]

    puts stdout ""
    puts stdout "Launching register .  .  ."
    puts stdout ""
    puts stdout ". . . starting at $StartTime . . ."
    puts $logfileId ""
    puts $logfileId "Launching register .  .  ."
    puts $logfileId ""
    puts $logfileId ". . . starting at $StartTime . . ."

    # if running on iris, spawn register on muir
    # I can't get register to compile correctly on iris 
    # plus it runs a lot quicker on muir
    if { [string compare "iris.nimh.nih.gov" [exec hostname]] == 0 } {
	
	set cmd " cd $funcdir; setenv DISPLAY iris.nimh.nih.gov; setenv SUBJECTS_DIR $SUBJECTS_DIR; $BINARIES2/register 2 $regstatinput $fthresh "
	set register_result [catch {exec $BINARIES2/remoteregister muir [format "%s" $cmd] >@ stdout } register_msg]
    } else {
	set register_result [catch {exec $BINARIES2/register 2 $regstatinput $fthresh >&@ stdout } register_msg]
    }
 
    close $logfileId
    
    return 0

}



proc PaintSurface { } {

    global BINARIES
    global BINARIES2
    global subjname
    global SUBJECTS_DIR
    global VERBOSE_FLAG
    global AFNIBIN
    global env

    set processdir [concat $SUBJECTS_DIR/$subjname]

    if [catch {open $processdir/MakeSurf.log a+ 0666} logfileId] {
	error "PaintSurface: Cannot open $processdir/MakeSurf.log: $logfileId"
    }

    # Assumption: the functional data exist in a directory separate from the 
    # surface directory. For now prompt the user to enter this functional directory

    puts stdout ""
    puts stdout "Enter the pathname of the functional directory"
    puts -nonewline stdout {: }
    flush stdout  ;#
    set funcdir [gets stdin]

    # check its existence
    if { ![file isdirectory $funcdir] } {
	error "PaintSurface: Functional directory, $funcdir, does not exist!"
    }

    cd $funcdir

    # check for existence of analyse.dat and register.dat
    # if register.dat exists but analyse.dat does not then remove register.dat and prompt for new info
    #if { ![file exists $funcdir/analyse.dat] && [file exists $funcdir/register.dat] } {	
    #}
    # if analyse.dat exists but register.dat does not then remove analyse.dat and prompt for new info
    #if { [file exists $funcdir/analyse.dat] && ![file exists $funcdir/register.dat] } {	
    #}
	
    # get the name of the floating point statistical map
    puts stdout ""
    puts stdout "Enter the name of a z-map to be painted to the surface"
    puts -nonewline stdout {: }
    flush stdout  ;#
    set imgfilename [gets stdin]

    # identify whether it is an ANALYZE or AFNI format file
    # the decision criteria are:
    # - if it ends in .hdr or .img, then it is an ANALYZE format image file
    # - if it ends in .HEAD or .BRIK then it is an AFNI format image file
    # after the file has been removed the tail is truncated and it is converted
    # to bshort MGH format for use with the register program

	
    if { [string match *.img [file tail $imgfilename]] } {
	set filetype ANALYZE
    }
    if { [string match *.hdr [file tail $imgfilename]] } {
	set filetype ANALYZE
    }
    if { [string match *.HEAD [file tail $imgfilename]] } {
	set filetype AFNI
    }
    if { [string match *.BRIK [file tail $imgfilename]] } {
	set filetype AFNI	
    }

    if { ![info exists filetype] } {
	error "PaintSurface: can't determine filetype for: $imgfilename"
    }

    switch -exact -- $filetype {
	
		
		ANALYZE {

		    puts stdout ""
		    puts stdout "Processing ANALYZE filetype"

		    set img_pathname [format %s%s [file rootname $imgfilename] .img]   
		    set hdr_pathname [format %s%s [file rootname $imgfilename] .hdr]

		    puts stdout "ANALYZE img name: $img_pathname"
		    puts stdout "ANALYZE hdr name: $hdr_pathname"

		    # read ANALYZE header information
		    set hdr_props [exec $BINARIES2/GetAVWInfo $hdr_pathname]
		    set num_props [llength $hdr_props]
	
		    if {$num_props != 14} {	
			error "PaintSurface: Cannot read header name: $hdr_pathname"	
		    }
	
		    set xdim [lindex $hdr_props 0]
		    set ydim [lindex $hdr_props 1]
		    set zdim [lindex $hdr_props 2]
		    set xvoxdim [lindex $hdr_props 3]
		    set yvoxdim [lindex $hdr_props 4]
		    set zvoxdim [lindex $hdr_props 5]
		    set bits [lindex $hdr_props 7]
		    set max [lindex $hdr_props 8]
		    set min [lindex $hdr_props 9]
		    set datatypecode [lindex $hdr_props 12]
	
		    set slicethickness $zvoxdim
		    set FOV [expr $xdim * $xvoxdim]
	
		    puts stdout ""
		    puts stdout "ANALYZE header contents for:"
		    puts stdout "$hdr_pathname"
		    puts stdout "X,Y,Z dimensions: $xdim $ydim $zdim"
		    puts stdout "X,Y,Z voxel sizes: $xvoxdim $yvoxdim $zvoxdim"
		    puts stdout "Maximum = $max, Minimum = $min"
		    puts stdout "Pixel encoding: $bits bits"
		    puts stdout "AVW datatypecode: $datatypecode"
		    puts stdout ""
    	
		    puts $logfileId ""
		    puts $logfileId "ANALYZE header contents for:"
		    puts $logfileId "$hdr_pathname"
		    puts $logfileId "X,Y,Z dimensions: $xdim $ydim $zdim"
		    puts $logfileId "X,Y,Z voxel sizes: $xvoxdim $yvoxdim $zvoxdim"
		    puts $logfileId "Maximum = $max, Minimum = $min"
		    puts $logfileId "Pixel encoding: $bits bits"
		    puts $logfileId "AVW datatypecode: $datatypecode"
		    puts $logfileId ""

		    # convert to MGH bfloat format

 
		    # finally we can call analyze2mgh
		    set stemname [file rootname $imgfilename]

		    puts stdout ""
		    puts stdout "Stemname: $stemname"
		    puts stdout ""

		    set avw2mgh_result [catch {exec $BINARIES2/analyze2mgh -i $stemname >&@ stdout} avw2mgh_msg]

		    # check to see if bfloats have been made correctly

		    # bfloat name goes here
		    set stem1 [format %s_ $stemname]
		    set stem2 "%03d"
		    set stem3 ".bfloat"
		    set paintinput $stem1$stem2$stem3
		    set rh_paintoutput [format %s_%s $stemname sigz-rh.w]
		    set lh_paintoutput [format %s_%s $stemname sigz-lh.w]

		    puts stdout ""
		    puts stdout "paint input name: $stem1$stem2$stem3"
		    puts stdout "rh output name: $rh_paintoutput"
		    puts stdout "lh output name: $lh_paintoutput"

		    # right hemisphere painting
		    set paint_result [catch {exec $BINARIES/paint $paintinput rh orig $rh_paintoutput -nslices $zdim  >&@ stdout } paint_msg]

		    # left hemisphere painting
		    set paint_result [catch {exec $BINARIES/paint $paintinput lh orig $lh_paintoutput -nslices $zdim  >&@ stdout } paint_msg]

		}

		AFNI {
		
		    puts stdout ""
		    puts stdout "Processing AFNI filetype"

		    set brik_pathname [format %s%s [file rootname $imgfilename] .BRIK]   
		    set head_pathname [format %s%s [file rootname $imgfilename] .HEAD]
		    
		    set stemname [file rootname $imgfilename]

		    puts stdout "AFNI BRIK name: $brik_pathname"
		    puts stdout "AFNI HEAD name: $head_pathname"

		    # check for existence of brik and head names
		    if { ![file exists $brik_pathname] } {
			error "PaintSurface: can't find $brik_pathname"
		    }
		    if { ![file exists $head_pathname] } {
			error "PaintSurface: can't find $head_pathname"
		    }

		    

		    # call ReadAFNIHEAD proc
		    set HEADInfo [ReadAFNIHEAD $head_pathname]

		    set afni_xdim [lindex $HEADInfo 0]
		    set afni_ydim [lindex $HEADInfo 1]
		    set afni_zdim [lindex $HEADInfo 2]
		    set afni_xvox [lindex $HEADInfo 3]
		    set afni_yvox [lindex $HEADInfo 4]
		    set afni_zvox [lindex $HEADInfo 5]
		    set afni_briksize [file size $brik_pathname]
		    set afni_planesize [expr $afni_briksize / $afni_zdim]

		    puts stdout ""
		    puts stdout "AFNI HEAD Information:"
		    puts stdout "X Dimension: $afni_xdim"
		    puts stdout "Y Dimension: $afni_ydim"
		    puts stdout "Z Dimension: $afni_zdim"
		    puts stdout "X Voxel Size: $afni_xvox"
		    puts stdout "Y Voxel Size: $afni_yvox"
		    puts stdout "Z Voxel Size: $afni_zvox"
		    puts stdout "Byte-size of Whole BRIK: $afni_briksize"
		    puts stdout "Byte-size of One 2D Slice: $afni_planesize"
		    puts stdout ""

		    puts stdout "Creating bfloat files from $brik_pathname . . . "
		    puts stdout ""
		    # can use the following to split
		    # -b codes the bytesize, -a means append only one letter after "tmp" for output files
		    set split_result [catch {exec split -b $afni_planesize -a 1 $brik_pathname tmp >&@ stdout} split_msg]

		    # rename output
		    # output will be in the current directory and will be tmp[a-z] if afni_zdim <= 26

		    if { $afni_zdim > 26} {
			error "PaintSurface: $brik_pathname has more than 26 slices, can't format split output."
		    } else {

			set letternum 1
			foreach letter {a b c d e f g h i j k l m n o p q r s t u v w x y z} {
			    
			    if { $letternum > $afni_zdim } {
				break
			    }

			    set outputblock [format tmp%s $letter]
			    set briktail [file tail $brik_pathname]
			    set bfloatblock [format %s_%03d.bfloat [file rootname $briktail] [expr $letternum - 1]]
			    set bfloathdr [format %s_%03d.hdr [file rootname $briktail] [expr $letternum - 1]]
			    exec mv $outputblock $bfloatblock

			    # open hdr file
			    if [catch {open $funcdir/$bfloathdr w} bfloathdr_fid] {
				error "PaintSurface: Can't open for writing: $funcdir/$bfloathdr"
			    }
			    # fill with bfloat image plane information
			    puts $bfloathdr_fid "$afni_xdim $afni_ydim 1 0"
			    # close hdr file
			    close $bfloathdr_fid 
			    incr letternum
			}
		    }

		    # bfloat name goes here
		    set stem1 [format %s_ $stemname]
		    set stem2 "%03d"
		    set stem3 ".bfloat"
		    set paintinput $stem1$stem2$stem3
		    set rh_paintoutput [format %s_%s $stemname sigz-rh.w]
		    set lh_paintoutput [format %s_%s $stemname sigz-lh.w]

		    puts stdout ""
		    puts stdout "paint input name: $stem1$stem2$stem3"
		    puts stdout "rh output name: $rh_paintoutput"
		    puts stdout "lh output name: $lh_paintoutput"

		    
		    # right hemisphere painting
		    set paint_result [catch {exec $BINARIES/paint $paintinput rh orig $rh_paintoutput -nslices $afni_zdim  >&@ stdout } paint_msg]

		    # left hemisphere painting
		    set paint_result [catch {exec $BINARIES/paint $paintinput lh orig $lh_paintoutput -nslices $afni_zdim  >&@ stdout } paint_msg]
		    

		}
		
		default {

		    error "Unidentified filetype: $filetype"
		
		
		}
		
	    }

    


    
   

    puts stdout ""
    puts stdout "Done painting"
    

   return 0

}

proc DisplaySurface { } {

    global BINARIES
    global subjname
    global SUBJECTS_DIR
    global VERBOSE_FLAG
    global AFNIBIN
    global env
	
    set processdir [concat $SUBJECTS_DIR/$subjname]

    if [catch {open $processdir/MakeSurf.log a+ 0666} logfileId] {
	error "DisplaySurface: Cannot open $processdir/MakeSurf.log: $logfileId"
    }
	
    # get the start time for processing
    set StartTime [exec date]

    # Assumption: the functional data exist in a directory separate from the 
    # surface directory. For now prompt the user to enter this functional directory

    puts stdout ""
    puts stdout "Enter the pathname of the functional directory"
    puts -nonewline stdout {: }
    flush stdout  ;#
    set funcdir [gets stdin]

    # check its existence
    if { ![file isdirectory $funcdir] } {
	error "DisplaySurface: Functional directory, $funcdir, does not exist!"
    }


    # show *.w file contents of the functional directory
    puts stdout ""
    puts stdout "The functional directory contains the following painted surface files:"
    puts stdout ""

    # get the list of I files
    set wfiles [glob -nocomplain $funcdir/*.w]
    if { [llength $wfiles] == 0 } {
	error "There are no *.w files in $funcdir"
    }

    foreach w $wfiles {
	puts stdout $w
    }
    puts stdout ""

    puts stdout ""
    puts stdout "Enter the name of the painted surface file you want to display:"
    puts -nonewline stdout {: }
    flush stdout  ;#
    set paintfile [gets stdin]

   

    puts stdout ""
    puts stdout "Enter a z-score threshold to use for the display:"
    puts -nonewline stdout {: }
    flush stdout  ;#
    set zthresh [gets stdin]

    # get the length in characters 
    set paintlength [string length $paintfile]
    # extract the hemisphere label
    set hemisphere [string range $paintfile [expr $paintlength - 4] [expr $paintlength - 3]]
    
    # let user choose folded, inflated or flattened surface display
    puts stdout ""
    puts stdout "Choose type of surface display:"
    puts stdout ""
    puts stdout "1: Folded Surface"
    puts stdout "2: Inflated Surface"
    puts stdout "3: Flattened Surface"
    puts stdout "4: Spherical Projection"
    puts stdout "5: Canonical Spherical Projection"
    puts stdout ""
    puts -nonewline stdout {: }
    flush stdout  ;#
    set surfaceinput [gets stdin]
    
    switch -exact -- $surfaceinput {

	1 { set surfacetype gray }
	2 { set surfacetype inflated }
	3 { set surfacetype inflated }
	4 { set surfacetype sphere }
	5 { set surfacetype sphere.reg }
	default { set surfacetype inflated }
    }
    

    # go to the scripts directory to launch tksurfer
    cd $SUBJECTS_DIR/$subjname/scripts

    puts stdout "Operating in: $SUBJECTS_DIR/$subjname/scripts"

    puts stdout ""
    puts stdout "Launching tksurfer .  .  ."
    puts stdout ""
    
    set funcdir func

    set hemi $hemisphere
    set env(noexit) 1
    set env(lat) 1
    set env(med) 1 
    set env(statflag) 1
    set env(statname) z
    set env(colscalebarflag) 1
    set env(scalebarflag) 1
    set env(colscale) 1
    set env(rgbname) test
    set env(dir) $funcdir
    set env(fthresh) $zthresh
    set env(w_file) $paintfile

    set env(fslope) 0.2
    set env(fmid) 3.0
    set env(smoothsteps) 10

    set display_result [catch {exec $BINARIES/tksurfer -$subjname $hemi $surfacetype -tcl /hippo/apps/flat/IRIX/lib/twocond-views.tcl & } display_msg]
    set tkmedit_result [catch {exec $BINARIES/tkmedit $subjname stat2T1 2 [format %s.orig $hemi] & } tkmedit_msg]

    return 0

}

#
# Utilities menu -
# allows manipulation of default parameters for surface
# processing algorithms
# 

proc SurfaceUtiltiesMenu { } {

        global SUBJECTS_DIR
	global subjname

	puts stdout ""
	puts stdout {******************************************************************}
	puts stdout {------------------------------------------------------------------}
	puts stdout {****************      Surface Utilities Menu      ****************}
	puts stdout ""
	puts stdout {1 : Change Default Parameters for Automated Skull Stripping       }
	puts stdout {2 : Change Default Parameters for White Matter Classification     }
	puts stdout {3 : Set Initialization Coordinates for mri_fill Cutting Planes    }
	puts stdout {4 : Toggle In-Plane Editing for tkmedit                           }
	puts stdout {5 : Return to Main Menu                                           }
	puts stdout {                                                                  }
	if {([info exists SUBJECTS_DIR] && [info exists subjname])} {
	  puts stdout "Current Working Surface Directory: $SUBJECTS_DIR/$subjname"
	}
	puts stdout {------------------------------------------------------------------}
	puts ""
	puts stdout ""
	puts stdout "Choose utility option:"
	puts -nonewline stdout {: }
	flush stdout  ;#
	set MenuChoice [gets stdin]
	
	set num_selected [llength $MenuChoice]
	
	if {$num_selected > 1} {
		return [lsort -integer $MenuChoice]
	} else {
		return $MenuChoice
	}
	
}

#
# SurfaceMenu -
# allows individual steps of the surface reconstruction
# to be implemented separately or all at once.
# 

proc SurfaceMenu { } {

        global SUBJECTS_DIR
	global subjname

	puts stdout ""
	puts stdout {******************************************************************}
	puts stdout {------------------------------------------------------------------}
	puts stdout {*****************      Surface Menu Options      *****************}
	puts stdout ""
	puts stdout {1 : Create Directory Hierarchy and Pre-Process Raw Anatomical Data}
	puts stdout {2 : Compute Talairach Transformation Matrix with MNI Autoreg      }
	puts stdout {3 : 3D Bias Correction                                            }
	puts stdout {4 : Automated Skull Removal			               }
	puts stdout {5 : Classify White Matter                                         }
	puts stdout {6 : Define Cutting Planes and Fill Hemispheres	               }
	puts stdout {7 : Tessellate and Smooth the Initial Right Hemisphere Surface    }
	puts stdout {8 : Tessellate and Smooth the Initial Left Hemisphere Surface     }
	puts stdout {9 : Inflate Right Hemisphere Surface Reconstruction               }
	puts stdout {10: Inflate Left Hemisphere Surface Reconstruction                }
        puts stdout {11: Edit Right Hemisphere Surface Reconstruction                  }
	puts stdout {12: Edit Left Hemisphere Surface Reconstruction                   }
        puts stdout {13: Reconstruct Final Right Hemisphere Surfaces                   }
        puts stdout {14: Reconstruct Final Left Hemisphere Surfaces                    }
	puts stdout {15: Flatten a Right Hemisphere Surface Reconstruction             }
	puts stdout {16: Flatten a Left Hemisphere Surface Reconstruction              }
	puts stdout {17: Align EPI or Coplanar Data to High Resolution Anatomy         }
	puts stdout {18: Paint Surface Reconstruction with Functional Data             }
	puts stdout {19: Display Painted Surface Reconstructions                       }
        puts stdout {20: Surface Utilities Menu                                        }
	puts stdout {21: Help                                                          }
	puts stdout {22: Exit                                                          }
	puts stdout {                                                                  }
	if {([info exists SUBJECTS_DIR] && [info exists subjname])} {
	  puts stdout "Current Working Surface Directory: $SUBJECTS_DIR/$subjname"
	}
	puts stdout {------------------------------------------------------------------}
	puts ""
	puts stdout ""
	puts stdout "Multiple operations may be run by inputing a string of integers."
	puts stdout "(e.g., to run operations 1 through 5 you would type: 1 2 3 4 5)"
	puts stdout ""

	puts -nonewline stdout "Choose option(s):"
	flush stdout  ;#
	set MenuChoice [gets stdin]
	
	set num_selected [llength $MenuChoice]
	
	if {$num_selected > 1} {
		return [lsort -integer $MenuChoice]
	} else {
		return $MenuChoice
	}
	
}

proc GetSurfaceHierarchy { } {

    global SUBJECTS_DIR
    global subjname
    global env

    puts stdout ""
    puts stdout "Please enter the surface analysis directory"
    puts stdout "that has been created to store the results."
    puts stdout "Note: You must enter the FULL pathname of"
    puts stdout "the upper level directory."

    puts -nonewline stdout {: }
    flush stdout  ;#
    set processdir [gets stdin]

    # get the tail of the processing directory
    # in Dale-speak this is the subjname variable

    set subjname [file tail $processdir]

    # get the directory path up to but not including
    # the last name. In Dale-speak this the SUBJECTS_DIR
    # variable which will be used as an environment variable

    set SUBJECTS_DIR [file dirname $processdir]

    set env(SUBJECTS_DIR) $SUBJECTS_DIR

    puts stdout ""

}

#
#
# End of Procedure definitions
#
#


# set the paths
# so they may 
# be used globally

SetSurfPaths
